/*****************************************************************************
 * @ingroup 1588
 *
 * @file 1588.c
 *
 * @description
 *        The 1588 driver module enables a client application to have
 *         access to the 1588 time synchronization hardware assist block.
 *         A client application can use this driver to retreive tx and rx timestamps
 *         of IEEE 1588(TM) PTP messages captured on ethernet or CAN ports.
 *         Access is provided by a set of APIs that can be called directly or through
 *         ioctl commands.
 *
 * This file is provided under a dual BSD/GPLv2 license.  When using or 
 *   redistributing this file, you may do so under either license.
 * 
 *   GPL LICENSE SUMMARY
 * 
 *   Copyright(c) 2007,2008 Intel Corporation. All rights reserved.
 * 
 *   This program is free software; you can redistribute it and/or modify 
 *   it under the terms of version 2 of the GNU General Public License as
 *   published by the Free Software Foundation.
 * 
 *   This program is distributed in the hope that it will be useful, but 
 *   WITHOUT ANY WARRANTY; without even the implied warranty of 
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 *   General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License 
 *   along with this program; if not, write to the Free Software 
 *   Foundation, Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *   The full GNU General Public License is included in this distribution 
 *   in the file called LICENSE.GPL.
 * 
 *   Contact Information:
 *   Intel Corporation
 * 
 *   BSD LICENSE 
 * 
 *   Copyright(c) 2007,2008 Intel Corporation. All rights reserved.
 *   All rights reserved.
 * 
 *   Redistribution and use in source and binary forms, with or without 
 *   modification, are permitted provided that the following conditions 
 *   are met:
 * 
 *     * Redistributions of source code must retain the above copyright 
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright 
 *       notice, this list of conditions and the following disclaimer in 
 *       the documentation and/or other materials provided with the 
 *       distribution.
 *     * Neither the name of Intel Corporation nor the names of its 
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 * 
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 *   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 *   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 *   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * 
 *  version: Embedded.L.1.0.34
 *
 ****************************************************************************/

#include "1588.h"

MODULE_AUTHOR("Intel(R) Corporation");
MODULE_DESCRIPTION("IEEE 1588* Time Synchronization Driver");
MODULE_LICENSE("Dual BSD/GPL");
MODULE_VERSION("1.0.0");

/*****************************************************************************
  Description:
    timesync_init is the entry point for the driver.

  Parameters:
    none

  Returns:
      0 => success
    < 0 => error
******************************************************************************/
int __init timesync_init(void)
{
    int ret;
    dev_t devno;

    memset(&g_drvr_data, 0, sizeof(drvr_data_t));

    /* request and reserve a device number */
    ret = alloc_chrdev_region(&g_drvr_data.devnum, 0, 1, DRIVERNAME);

    if ( ret < 0)
    {
      printk("%s-timesync_init: Could not register module\n", DRIVERNAME);
      return ret;
    }

    /* init cdev struct for adding device to kernel */
    cdev_init(&g_drvr_data.cdev, &file_ops);
    g_drvr_data.cdev.owner = THIS_MODULE;
    g_drvr_data.cdev.ops = &file_ops;

    devno = MKDEV(MAJOR(g_drvr_data.devnum), 0);

    if ( cdev_add(&g_drvr_data.cdev, devno, 1) )
    {
        printk("%s-timesync_init: cdev_add failed\n", DRIVERNAME);
        goto exit_error;
    }

    /* register the driver with the pci core*/
    ret = pci_register_driver(&pci_ops);

    if ( ret )
    {
        printk("%s-timesync_init: pci_register failed\n", DRIVERNAME);
      goto exit_error;
    }

    goto exit;

exit_error:
    printk("%s-timesync_init: Initialization failed\n", DRIVERNAME);
    timesync_close();
    return -ENODEV;

exit:
    printk("%s-timesync_init: Initialization complete\n", DRIVERNAME);
    return 0;
}

/*****************************************************************************
  Description:
    timesync_close is called when the driver is unloaded from memory.

  Parameters:
    none

  Returns:
      none
******************************************************************************/
void timesync_close(void)
{
    /* remove cdev struct from system */
    cdev_del(&g_drvr_data.cdev);

    unregister_chrdev_region(g_drvr_data.devnum, 1);

    pci_unregister_driver(&pci_ops);

    printk("%s-timesync_close: Driver unload complete\n", DRIVERNAME);
}


/*****************************************************************************
  Description:
    timesync_open is called when the driver interface is opened

  Parameters:
    inode  kernel file structure
    filp   open file structure

  Returns:
      0 => success
    < 0 => error
******************************************************************************/
int timesync_open(struct inode *inode, struct file *filp)
{
    if ( g_drvr_data.suspend )
    {
      return -EINTR;
    }

    printk("%s-timesync_open: module opened\n", DRIVERNAME);

    return 0;
}

/*****************************************************************************
  Description:
    timesync_release is called when the driver interface is closed

  Parameters:
    inode  kernel file structure
    filp   open file structure

  Returns:
      0 => success
    < 0 => error

******************************************************************************/
int timesync_release(struct inode *inode, struct file *filp)
{
    printk("%s-timesync_release: module released\n", DRIVERNAME);

    return 0;
}

/*****************************************************************************
  Description:
    pci_probe is called right after insmod by the PCI core. Here we enable the
    device to make the device's resources live. We can then read PCI CFG space
    and init the device.

  Parameters:
    dev   pci device structure
    id    types of devices supported

  Returns:
      0 => success
    < 0 => error

******************************************************************************/
int pci_probe(struct pci_dev *dev, const struct pci_device_id *id)
{

    int ret = 0;
    int i;

    g_drvr_data.mem_base = pci_resource_start(dev, IO_MEM_BAR);

    if ( !g_drvr_data.mem_base )
    {
        printk("%s-pci_probe: could not locate IO memory address\n",
            DRIVERNAME);
        goto exit_error;
    }

    /* retreive the available length of the IO memory space */
    g_drvr_data.mem_size = pci_resource_len(dev, IO_MEM_BAR);

    printk("%s-pci_probe: dev memory len %d\n", DRIVERNAME,
        g_drvr_data.mem_size);

    /* allocate the memory for the gpio registers */
    if ( !request_mem_region(g_drvr_data.mem_base, g_drvr_data.mem_size,
                "1588_regs") )
    {
        printk("%s-pci_probe: could not allocate register memory space\n",
            DRIVERNAME);
        g_drvr_data.mem_base = 0;
        goto exit_error;
    }

    /* get the virtual address to the 1588 registers */
    g_drvr_data.mem_virt = ioremap(g_drvr_data.mem_base, g_drvr_data.mem_size);

    if ( !g_drvr_data.mem_virt )
    {
        printk("%s-pci_probe: Could not get virtual address\n", DRIVERNAME);
        goto exit_error;
    }

    /* enable the 1588 pci device */
    ret = pci_enable_device(dev);

    if ( ret )
    {
      printk("%s-pci_probe: pci_enable_device failed\n", DRIVERNAME);
      return ret;
    }

    for (i=0; i < NUM_EVENTS; i++)
    {
      init_waitqueue_head(&g_notify_event[i]);
      g_eventflag[i] = 0;
    }

    TargetTimeCallbackPtr = TargTimeCallback;
    AuxTimeCallbackPtr = AuxTimeCallback;
    AuxTargetTimeCallbackPtr = AuxTargetTimeCallback;
    PulsePerSecondCallbackPtr = PulsePerSecondCallback;

    if ( ixTimeSyncAccBlPlBaseAddressesSet((unsigned int)g_drvr_data.mem_virt)
        != ICP_TIMESYNCACC_SUCCESS )
    {
      printk("%s-pci_probe: ixTimeSyncAccBlPlBaseAddressesSet failed\n",
          DRIVERNAME);
      goto exit_error;

    }

    if ( request_irq(dev->irq, &timesync_isr, SA_SHIRQ, DRIVERNAME,
         &g_drvr_data) )
    {
      printk("%s-pci_probe: irq\n", DRIVERNAME);
      goto exit_error;
    }

    /* indicate success */
    g_drvr_data.irq = dev->irq;

    /* enable interrupts to be sent to IA cpu */
    if ( pci_write_config_byte(dev, 0xE8, 1) )
    {
      printk("%s-pci_probe: pci_write_config_byte failed\n", DRIVERNAME);
      goto exit_error;
    }

    goto exit;

exit_error:

    printk("%s-pci_probe: probe failed\n", DRIVERNAME);
    return -ENODEV;

exit:

    printk("%s-pci_probe: probe succeeded\n", DRIVERNAME);
    return 0;
}

/*****************************************************************************
  Description:
    pci_remove is called when the pci driver is being unloaded from the
    kernel.

  Parameters:
    dev   pci device structure

  Returns:
      0 => success
    < 0 => error

******************************************************************************/
void pci_remove(struct pci_dev *dev)
{
    /* disable the interrupts on the 1588 hardware */
    disable_interrupts();

    /* free the interrupt */
    if ( dev->irq )
    {
      free_irq(dev->irq, &g_drvr_data);
    }

    pci_disable_device(dev);

    /* unmap the virtual IO memory space */
    if ( g_drvr_data.mem_virt )
    {
        iounmap(g_drvr_data.mem_virt);
    }

    /* release the reserved IO memory space */
    if ( g_drvr_data.mem_base )
    {
        release_mem_region(g_drvr_data.mem_base, g_drvr_data.mem_size);
    }

    printk("%s-pci_remove: remove complete\n", DRIVERNAME);
}

/*****************************************************************************
  Description:
    pci_suspend is called to suspend a device before being put into a
    low power state.

  Parameters:
    dev    pci device structure
    state  suspend state

  Returns:
      0 => success
    < 0 => error

******************************************************************************/
int pci_suspend(struct pci_dev *dev, pm_message_t state)
{

    disable_interrupts();

    g_drvr_data.suspend = 1;

    pci_disable_device(dev);

    save_reg_state();

    pci_save_state(dev);

    if ( pci_set_power_state(dev, pci_choose_state(dev, state)) == -EIO )
    {
      printk("%s-pci_suspend: does not support PM cpabilities\n", DRIVERNAME);
    }

    return 0;
}

/*****************************************************************************
  Description:
    pci_resume is called to resume a device after being put into a
    low power state.

  Parameters:
    dev   pci device structure

  Returns:
      0 => success
    < 0 => error

******************************************************************************/
int pci_resume(struct pci_dev *dev)
{
    int ret;

    ret = pci_set_power_state(dev, PCI_D0);
    if ( ret ) {
      printk("%s-pci_resume: does not support PM cpabilities\n", DRIVERNAME);
    }

    pci_restore_state(dev);

    restore_reg_state();

    ret = pci_enable_device(dev);

    if ( ret )
    {
      printk("%s-pci_resume: pci_enable_device failed\n", DRIVERNAME);
      return ret;
    }

    g_drvr_data.suspend = 0;

    /* IXA00160934 - 1588 device not seeing interrupts after S3 resume
     * due to SMIA (0xE8) not being set to 1 by driver.
     */
    /* enable interrupts to be sent to IA cpu */
    ret = pci_write_config_byte(dev, 0xE8, 1);
    if ( ret )
    {
      printk("%s-pci_resume: pci_write_config_byte to SMIA failed, "
             "interrupts not routed to CPU\n", DRIVERNAME);
      return ret;
    }

    restore_interrupts();

    return 0;
}


/*****************************************************************************
  Description:
    save_reg_state is called to save device registers before being put into a
    low power state.

  Parameters:
    none

  Returns:
      0 => success
    < 0 => error

******************************************************************************/
int save_reg_state(void)
{
    g_drvr_data.state.AmmsInterrupt=
                        ixTimeSyncAccControlAmmsInterruptMaskGet();
    g_drvr_data.state.AsmsInterrupt=
                        ixTimeSyncAccControlAsmsInterruptMaskGet();
    g_drvr_data.state.TtmInterrupt=
                        ixTimeSyncAccControlTtmInterruptMaskGet();
    g_drvr_data.state.AtmInterrupt=
                        ixTimeSyncAccControlAtmInterruptMaskGet();
    g_drvr_data.state.PpsmInterrupt=
                        ixTimeSyncAccControlPpsmInterruptMaskGet();
    ixTimeSyncAccSystemTimeSnapshotGet(
                    &g_drvr_data.state.SystemTime.timeValueLowWord,
                    &g_drvr_data.state.SystemTime.timeValueHighWord);
    ixTimeSyncAccTargetTimeSnapshotGet(
                    &g_drvr_data.state.TargetTime.timeValueLowWord,
                    &g_drvr_data.state.TargetTime.timeValueHighWord);
    ixTimeSyncAccAuxTargetTimeSnapshotGet(
                    &g_drvr_data.state.AuxTargetTime.timeValueLowWord,
                    &g_drvr_data.state.AuxTargetTime.timeValueHighWord);
    ixTimeSyncAccPulsePerSecondCompareGet(&g_drvr_data.state.PulsePerSecond);
    ixTimeSyncAccAddendFsvGet(&g_drvr_data.state.Addend);

    g_drvr_data.state.MasterMode[ICP_TIMESYNCACC_GBE_0_1588PTP_PORT] =
      ixTimeSyncAccControlPTPPortMasterModeGet(
                ICP_TIMESYNCACC_GBE_0_1588PTP_PORT);
    g_drvr_data.state.MasterMode[ICP_TIMESYNCACC_GBE_1_1588PTP_PORT] =
      ixTimeSyncAccControlPTPPortMasterModeGet(
                ICP_TIMESYNCACC_GBE_1_1588PTP_PORT);

    g_drvr_data.state.MsgTimestamp[ICP_TIMESYNCACC_GBE_0_1588PTP_PORT] =
      ixTimeSyncAccControlPTPPortPTPMsgTimestampGet(
                ICP_TIMESYNCACC_GBE_0_1588PTP_PORT);
    g_drvr_data.state.MsgTimestamp[ICP_TIMESYNCACC_GBE_1_1588PTP_PORT] =
      ixTimeSyncAccControlPTPPortPTPMsgTimestampGet(
                ICP_TIMESYNCACC_GBE_1_1588PTP_PORT);

    return 0;
}


/*****************************************************************************
  Description:
    restore_reg_state is called to restore device registers when being put
    into a full power state.

  Parameters:
    none

  Returns:
      0 => success
    < 0 => error

******************************************************************************/
int restore_reg_state(void)
{
    ixTimeSyncAccTargetTimeSnapshotSet(
        g_drvr_data.state.TargetTime.timeValueLowWord,
        g_drvr_data.state.TargetTime.timeValueHighWord);
    ixTimeSyncAccAuxTargetTimeSnapshotSet(
        g_drvr_data.state.AuxTargetTime.timeValueLowWord,
        g_drvr_data.state.AuxTargetTime.timeValueHighWord);
    ixTimeSyncAccPulsePerSecondCompareSet(g_drvr_data.state.PulsePerSecond);
    ixTimeSyncAccAddendFsvSet(g_drvr_data.state.Addend);

    ixTimeSyncAccControlPTPPortMasterModeSet(
        ICP_TIMESYNCACC_GBE_0_1588PTP_PORT,
        g_drvr_data.state.MasterMode[ICP_TIMESYNCACC_GBE_0_1588PTP_PORT]);
    ixTimeSyncAccControlPTPPortMasterModeSet(
        ICP_TIMESYNCACC_GBE_1_1588PTP_PORT,
        g_drvr_data.state.MasterMode[ICP_TIMESYNCACC_GBE_1_1588PTP_PORT]);

    ixTimeSyncAccControlPTPPortPTPMsgTimestampSet(
        ICP_TIMESYNCACC_GBE_0_1588PTP_PORT,
        g_drvr_data.state.MsgTimestamp[ICP_TIMESYNCACC_GBE_0_1588PTP_PORT]);
    ixTimeSyncAccControlPTPPortPTPMsgTimestampSet(
        ICP_TIMESYNCACC_GBE_1_1588PTP_PORT,
        g_drvr_data.state.MsgTimestamp[ICP_TIMESYNCACC_GBE_1_1588PTP_PORT]);

    return 0;
}


/*****************************************************************************
  Description:
    disable_interrupts disables all interrupts on the 1588 device.

  Parameters:
    none

  Returns:
      0 => success
    < 0 => error

******************************************************************************/
int disable_interrupts(void)
{
    ixTimeSyncAccControlAmmsInterruptMaskClear();
    ixTimeSyncAccControlAsmsInterruptMaskClear();
    ixTimeSyncAccControlTtmInterruptMaskClear();
    ixTimeSyncAccControlAtmInterruptMaskClear();
    ixTimeSyncAccControlPpsmInterruptMaskClear();

    return 0;
}


/*****************************************************************************
  Description:
    restore_interrupts enables interrupts which were previously disabled
    on the 1588 device.

  Parameters:
    none

  Returns:
      0 => success
    < 0 => error

******************************************************************************/
int restore_interrupts(void)
{
    if ( g_drvr_data.state.AmmsInterrupt )
    {
      ixTimeSyncAccControlAmmsInterruptMaskSet();
    }

    if ( g_drvr_data.state.AsmsInterrupt )
    {
      ixTimeSyncAccControlAsmsInterruptMaskSet();
    }

    if ( g_drvr_data.state.TtmInterrupt )
    {
      ixTimeSyncAccControlTtmInterruptMaskSet();
    }

    if ( g_drvr_data.state.AtmInterrupt )
    {
      ixTimeSyncAccControlAtmInterruptMaskSet();
    }

    if ( g_drvr_data.state.PpsmInterrupt )
    {
      ixTimeSyncAccControlPpsmInterruptMaskSet();
    }

    return 0;
}



/*****************************************************************************
  Description:
    timesync_isr is the driver interrupt service routine.

  Parameters:
    irq     interrupt number
    dev_id  client data
    regs    processor context

  Returns:
      IRQ_HANDLED => interrupt handled
      IRQ_NONE => this device did not interrupt

******************************************************************************/
irqreturn_t timesync_isr(int irq, void *dev_id, struct pt_regs *regs)
{
    if ( !ixTimeSyncAccEventAmmsFlagGet() && !ixTimeSyncAccEventAsmsFlagGet()&&
       !ixTimeSyncAccEventAtmFlagGet() && !ixTimeSyncAccEventPpsmFlagGet()&&
       !ixTimeSyncAccEventTtmFlagGet() )
    {
      return IRQ_NONE;
    }

    if ( ixTimeSyncAccIsr() != ICP_TIMESYNCACC_SUCCESS )
    {
      printk("%s-timesync_isr: ixTimeSyncAccIsr failed\n", DRIVERNAME);
      return IRQ_NONE;
    }

    return IRQ_HANDLED;
}

/*****************************************************************************
  Description:
    timesync_ioctl is the ioctl interface to the driver.

  Parameters:
    inode  ptr to inode
    filp   ptr to file struct
    cmd    ioctl command
    arg    passed in argument

  Returns:
      0 => success
    < 0 => error
******************************************************************************/
int timesync_ioctl(struct inode *inode, struct file *filp, unsigned int cmd,
                unsigned long arg)
{
    wait_queue_head_t *event = NULL;
    unsigned int bytes_ret = 0;
    void *param_addr = NULL;
    unsigned int eventnum = 0;
    unsigned int buffsize;
    unsigned int val32;
    char buff[0x64];
    int ret = 0;
    int i = 0;

    IxTimeSyncAccAuxMode auxMode;
    IxTimeSyncAcc1588PTPPort ptpPort;
    IxTimeSyncAccPortCfgIoctl *pPortCfgIoctl;
    IxTimeSyncAccRxTxPollIoctl *pRxTxPollIoctl;
    IxTimeSyncAccCANPollIoctl *pCANPollIoctl;
    IxTimeSyncAccTimePollIoctl *pTimePollIoctl;
    IxTimeSyncAccTimeValue TimeValue;
    IxTimeSyncAccVersionIoctl *pVersionIoctl;
    IxTimeSyncAccOperationModeIoctl *pOperationModeIoctl;


    if ( g_drvr_data.suspend )
    {
      return -EINTR;
    }

    buffsize = _IOC_SIZE(cmd);

    if ( buffsize > sizeof(buff) )
    {
      printk("%s-timesync_ioctl: buffer size too small.\n", DRIVERNAME);
      return -EINVAL;
    }

    /* if data is being written to the driver */
    if ( _IOC_DIR(cmd) & _IOC_WRITE )
    {
        /* get the data passed in by user */
        if ( copy_from_user(&buff, (void*)arg, buffsize) )
        {
            printk("%s-timesync_ioctl: could not copy user space data.\n",
                DRIVERNAME);
            return -EFAULT;
        }
    }

    switch ( cmd )
    {
      case IOCTL_TARG_TIME_NOTIFY:
      case IOCTL_AUX_TIME_NOTIFY:
      case IOCTL_AUX_TARG_TIME_NOTIFY:
      case IOCTL_PULSE_PER_SEC_NOTIFY:

           /*
            *
            * request to be notified of a 1588 interrupt event

            Target Time
            */

            if ( cmd == IOCTL_TARG_TIME_NOTIFY )
            {
                event = &g_notify_event[TARG_TIME_EVENT_NUM];
                bytes_ret = sizeof(IxTimeSyncAccTimeValue);
                param_addr = &gTargTimeIoctl;
                eventnum = TARG_TIME_EVENT_NUM;
            }
            else if ( cmd == IOCTL_AUX_TIME_NOTIFY )
            {
                event = &g_notify_event[AUX_TIME_EVENT_NUM];
                bytes_ret = sizeof(IxTimeSyncAccAuxTimeIoctl);
                param_addr = &gAuxTimeIoctl;
                eventnum = AUX_TIME_EVENT_NUM;
            }
            else if ( cmd == IOCTL_AUX_TARG_TIME_NOTIFY )
            {
                event = &g_notify_event[AUX_TARG_TIME_EVENT_NUM];
                bytes_ret = sizeof(IxTimeSyncAccTimeValue);
                param_addr = &gAuxTargTimeIoctl;
                eventnum = AUX_TARG_TIME_EVENT_NUM;
            }
            else /* if ( cmd == IOCTL_PULSE_PER_SEC_NOTIFY ) */
            {
                event = &g_notify_event[PPS_EVENT_NUM];
                bytes_ret = sizeof(UINT32);
                param_addr = &gPulsePerSecondIoctl;
                eventnum = PPS_EVENT_NUM;
            }

        g_eventflag[eventnum] = 0;

        /* wait infinitely for a 1588 interrupt event to occur */
        wait_event_interruptible(*event, g_eventflag[eventnum] == 1);

        /* copy global data retreived from interrupt handler */
        memcpy(&buff[0], param_addr, bytes_ret);

        /* reset global data to 0 */
        memset(param_addr, 0, bytes_ret);

        g_eventflag[eventnum] = 0;

            break;

      case IOCTL_TARG_TIME_CLR_NOTIFY:
      case IOCTL_AUX_TIME_CLR_NOTIFY:
      case IOCTL_AUX_TARG_TIME_CLR_NOTIFY:
      case IOCTL_PULSE_PER_SEC_CLR_NOTIFY:

           /*
            *
            * request to release a notify thread that is waiting
            * on a 1588 interrupt event
            *

            * Target Time
            */

            if ( cmd == IOCTL_TARG_TIME_CLR_NOTIFY )
            {
            eventnum = TARG_TIME_EVENT_NUM;
            }
            else if ( cmd == IOCTL_AUX_TIME_CLR_NOTIFY )
            {
            eventnum = AUX_TIME_EVENT_NUM;
            }
            else if ( cmd == IOCTL_AUX_TARG_TIME_CLR_NOTIFY )
            {
                eventnum = AUX_TARG_TIME_EVENT_NUM;
            }
            else if ( cmd == IOCTL_PULSE_PER_SEC_CLR_NOTIFY )
            {
                eventnum = PPS_EVENT_NUM;
            }

        g_eventflag[eventnum] = 1;

        wake_up_interruptible(&g_notify_event[eventnum]);
            break;

      case IOCTL_TARG_TIME_INTRPT_ENABLE:
            if ( ixTimeSyncAccTargetTimeInterruptEnable(TargetTimeCallbackPtr)
                    != ICP_TIMESYNCACC_SUCCESS )
            {
                printk("%s-timesync_ioctl: \
                    ixTimeSyncAccTargetTimeInterruptEnable failed\n",
                        DRIVERNAME);
                ret = -EINVAL;
            }
            break;

      case IOCTL_AUX_TIME_INTRPT_ENABLE:
        memcpy(&auxMode, buff, sizeof(IxTimeSyncAccAuxMode));

            if ( ixTimeSyncAccAuxTimeInterruptEnable(
                auxMode, AuxTimeCallbackPtr)
                    != ICP_TIMESYNCACC_SUCCESS )
            {
                printk("%s-timesync_ioctl: \
                    ixTimeSyncAccAuxTimeInterruptEnable failed\n",
                        DRIVERNAME);
                ret = -EINVAL;
            }
            break;

      case IOCTL_AUX_TARG_TIME_INTRPT_ENABLE:
            if ( ixTimeSyncAccAuxTargetTimeInterruptEnable(
                AuxTargetTimeCallbackPtr)
                    != ICP_TIMESYNCACC_SUCCESS )
            {
                printk("%s-timesync_ioctl: \
                    ixTimeSyncAccAuxTargetTimeInterruptEnable failed\n",
                        DRIVERNAME);
                ret = -EINVAL;
            }
            break;

      case IOCTL_PULSE_PER_SEC_INTRPT_ENABLE:
            if ( ixTimeSyncAccPulsePerSecondInterruptEnable(
                PulsePerSecondCallbackPtr)
                != ICP_TIMESYNCACC_SUCCESS )
            {
                printk("%s-timesync_ioctl: \
                    ixTimeSyncAccPulsePerSecondInterruptEnable failed\n",
                        DRIVERNAME);
                ret = -EINVAL;
            }
            break;

      case IOCTL_TARG_TIME_INTRPT_DISABLE:
            if ( ixTimeSyncAccTargetTimeInterruptDisable()
                    != ICP_TIMESYNCACC_SUCCESS )
            {
                printk("%s-timesync_ioctl: \
                    ixTimeSyncAccTargetTimeInterruptDisable failed\n",
                        DRIVERNAME);
                ret = -EINVAL;
            }
            break;

      case IOCTL_AUX_TIME_INTRPT_DISABLE:
        memcpy(&auxMode, buff, sizeof(IxTimeSyncAccAuxMode));

            if ( ixTimeSyncAccAuxTimeInterruptDisable(auxMode)
                    != ICP_TIMESYNCACC_SUCCESS )
            {
                printk("%s-timesync_ioctl: \
                    ixTimeSyncAccAuxTimeInterruptDisable failed\n",
                        DRIVERNAME);
                ret = -EINVAL;
            }
            break;

      case IOCTL_AUX_TARG_TIME_INTRPT_DISABLE:

            if ( ixTimeSyncAccAuxTargetTimeInterruptDisable()
                    != ICP_TIMESYNCACC_SUCCESS )
            {
                printk("%s-timesync_ioctl: \
                    ixTimeSyncAccAuxTargetTimeInterruptDisable failed\n",
                        DRIVERNAME);
                ret = -EINVAL;
            }
            break;

      case IOCTL_PULSE_PER_SEC_INTRPT_DISABLE:

            if ( ixTimeSyncAccPulsePerSecondInterruptDisable()
                    != ICP_TIMESYNCACC_SUCCESS )
            {
                printk("%s-timesync_ioctl: \
                    ixTimeSyncAccPulsePerSecondInterruptDisable failed\n",
                        DRIVERNAME);
                ret = -EINVAL;
            }
            break;

      case IOCTL_PORT_CONFIG_SET:

        pPortCfgIoctl = (IxTimeSyncAccPortCfgIoctl*)buff;

            if ( ixTimeSyncAccPTPPortConfigSet(pPortCfgIoctl->ptpPort,
                               pPortCfgIoctl->ptpPortMode)
                    != ICP_TIMESYNCACC_SUCCESS )
            {
                printk("%s-timesync_ioctl: \
                    ixTimeSyncAccPTPPortConfigSet failed\n",
                        DRIVERNAME);
                ret = -EINVAL;
            }
            break;

      case IOCTL_PORT_CONFIG_GET:

            pPortCfgIoctl = (IxTimeSyncAccPortCfgIoctl*)buff;

            if ( ixTimeSyncAccPTPPortConfigGet(pPortCfgIoctl->ptpPort,
                               &pPortCfgIoctl->ptpPortMode)
                    != ICP_TIMESYNCACC_SUCCESS )
            {
                printk("%s-timesync_ioctl: \
                    ixTimeSyncAccPTPPortConfigGet failed\n",
                        DRIVERNAME);
                ret = -EINVAL;
            }
        break;

      case IOCTL_RX_POLL:
      case IOCTL_TX_POLL:
        pRxTxPollIoctl = (IxTimeSyncAccRxTxPollIoctl*)buff;
        if ( cmd == IOCTL_RX_POLL )
        {
                if ( ixTimeSyncAccPTPRxPoll(pRxTxPollIoctl->ptpPort,
                            &pRxTxPollIoctl->ptpMsgData)
                        != ICP_TIMESYNCACC_SUCCESS )
                {
                    printk("%s-timesync_ioctl: ixTimeSyncAccPTPRxPoll "
                        "failed\n",
                            DRIVERNAME);
                    ret = -EINVAL;
                }
        }
        else if ( cmd == IOCTL_TX_POLL )
        {
                if ( ixTimeSyncAccPTPTxPoll(pRxTxPollIoctl->ptpPort,
                            &pRxTxPollIoctl->ptpMsgData)
                        != ICP_TIMESYNCACC_SUCCESS )
                {
                    printk("%s-timesync_ioctl: ixTimeSyncAccPTPTxPoll "
                        "failed\n",
                            DRIVERNAME);
                    ret = -EINVAL;
                }
        }
            break;

      case IOCTL_CAN_POLL:
        pCANPollIoctl = (IxTimeSyncAccCANPollIoctl*)buff;
            if ( ixTimeSyncAccPTPCANPoll(pCANPollIoctl->ptpPort,
                           &pCANPollIoctl->ptpTimeStamp)
                    != ICP_TIMESYNCACC_SUCCESS )
            {
                printk("%s-timesync_ioctl: ixTimeSyncAccPTPCANPoll failed\n",
                        DRIVERNAME);
                ret = -EINVAL;
            }
        break;

      case IOCTL_SYS_TIME_SET:
      case IOCTL_TARG_TIME_SET:
      case IOCTL_AUX_TARG_TIME_SET:
        memcpy(&TimeValue, buff, sizeof(IxTimeSyncAccTimeValue));

            if ( cmd == IOCTL_SYS_TIME_SET )
        {
                if ( ixTimeSyncAccSystemTimeSet(TimeValue)
                      != ICP_TIMESYNCACC_SUCCESS )
                {
                  printk("%s-timesync_ioctl: \
                      ixTimeSyncAccSystemTimeSet failed\n",
                          DRIVERNAME);
                  ret = -EINVAL;
                }
        }
        /* set target time */
        else if ( cmd == IOCTL_TARG_TIME_SET )
        {
                if ( ixTimeSyncAccTargetTimeSet(TimeValue)
                      != ICP_TIMESYNCACC_SUCCESS )
                {
                  printk("%s-timesync_ioctl: \
                      ixTimeSyncAccTargetTimeSet failed\n",
                          DRIVERNAME);
                  ret = -EINVAL;
                }
        }

            else if ( cmd == IOCTL_AUX_TARG_TIME_SET )
        {
                if ( ixTimeSyncAccAuxTargetTimeSet(TimeValue)
                      != ICP_TIMESYNCACC_SUCCESS )
                {
                  printk("%s-timesync_ioctl: \
                      ixTimeSyncAccAuxTargetTimeSet failed\n",
                          DRIVERNAME);
                  ret = -EINVAL;
                }
        }
        break;

      case IOCTL_SYS_TIME_GET:
      case IOCTL_TARG_TIME_GET:
      case IOCTL_AUX_TARG_TIME_GET:

        TimeValue = *(IxTimeSyncAccTimeValue*)buff;

        if ( cmd == IOCTL_SYS_TIME_GET )
        {
                if ( ixTimeSyncAccSystemTimeGet(&TimeValue)
                      != ICP_TIMESYNCACC_SUCCESS )
                {
                  printk("%s-ixTimeSyncAccSystemTimeGet failed\n",
                      DRIVERNAME);
                  ret = -EINVAL;
              break;
                }
        }

            else if ( cmd == IOCTL_TARG_TIME_GET )
        {
                if ( ixTimeSyncAccTargetTimeGet(&TimeValue)
                      != ICP_TIMESYNCACC_SUCCESS )
                {
                  printk("%s-timesync_ioctl: \
                      ixTimeSyncAccTargetTimeGet failed\n",
                          DRIVERNAME);
                  ret = -EINVAL;
              break;
                }
        }

            else if ( cmd == IOCTL_AUX_TARG_TIME_GET )
        {
                if ( ixTimeSyncAccAuxTargetTimeGet(&TimeValue)
                      != ICP_TIMESYNCACC_SUCCESS )
                {
                  printk("%s-timesync_ioctl: \
                      ixTimeSyncAccAuxTargetTimeGet failed\n",
                          DRIVERNAME);
                  ret = -EINVAL;
              break;
                }
        }


            memcpy(buff, &TimeValue, sizeof(IxTimeSyncAccTimeValue));

        break;

      case IOCTL_TICK_RATE_GET:
      case IOCTL_TICK_RATE_SET:
      case IOCTL_PULSE_PER_SEC_TIME_SET:
      case IOCTL_PULSE_PER_SEC_TIME_GET:

            if ( cmd == IOCTL_TICK_RATE_GET )
        {
                if ( ixTimeSyncAccTickRateGet(&val32)
                      != ICP_TIMESYNCACC_SUCCESS )
                {
                  printk("%s-timesync_ioctl: \
                      ixTimeSyncAccTickRateGet failed\n",
                          DRIVERNAME);
                  ret = -EINVAL;
                }

            memcpy(buff, &val32, sizeof(UINT32));
        }

            else if ( cmd == IOCTL_TICK_RATE_SET )
        {
            memcpy(&val32, (UINT32*)&buff[0], sizeof(UINT32));

                if ( ixTimeSyncAccTickRateSet(val32)
                      != ICP_TIMESYNCACC_SUCCESS )
                {
                  printk("%s-timesync_ioctl: \
                      ixTimeSyncAccTickRateSet failed\n",
                          DRIVERNAME);
                  ret = -EINVAL;
                }
            }

            else if ( cmd == IOCTL_PULSE_PER_SEC_TIME_GET )
        {
                if ( ixTimeSyncAccPulsePerSecondTimeGet(&val32)
                      != ICP_TIMESYNCACC_SUCCESS )
                {
                    printk("%s-timesync_ioctl: \
                        ixTimeSyncAccPulsePerSecondTimeGet failed\n",
                                DRIVERNAME);
                  ret = -EINVAL;
                }

            memcpy(&buff[0], &val32, sizeof(UINT32));
        }

            else if ( cmd == IOCTL_PULSE_PER_SEC_TIME_SET )
        {
            memcpy(&val32, (UINT32*)&buff[0], sizeof(UINT32));

                if ( ixTimeSyncAccPulsePerSecondTimeSet(val32)
                                != ICP_TIMESYNCACC_SUCCESS )
                {
                    printk("%s-timesync_ioctl: \
                        ixTimeSyncAccPulsePerSecondTimeSet failed\n",
                                DRIVERNAME);
                  ret = -EINVAL;
                }
        }
        break;

      case IOCTL_TARG_TIME_POLL:
      case IOCTL_AUX_TARG_TIME_POLL:
      case IOCTL_AUX_TIME_POLL:

           /*
            *
            * request to poll the time status of the 1588 timers
            *

            * cast the user buff to a TimePollIoctl struct
            */

        pTimePollIoctl = (IxTimeSyncAccTimePollIoctl*)buff;

        if ( cmd == IOCTL_TARG_TIME_POLL )
        {
                if ( ixTimeSyncAccTargetTimePoll(&pTimePollIoctl->PollFlag,
                                 &pTimePollIoctl->timeVal)
                      != ICP_TIMESYNCACC_SUCCESS )
                {
                  printk("%s-timesync_ioctl: \
                      ixTimeSyncAccTargetTimePoll failed\n",
                          DRIVERNAME);
                  ret = -EINVAL;
                }
        }
        else if ( cmd == IOCTL_AUX_TARG_TIME_POLL )
        {
                if ( ixTimeSyncAccAuxTargetTimePoll(&pTimePollIoctl->PollFlag,
                                  &pTimePollIoctl->timeVal)
                      != ICP_TIMESYNCACC_SUCCESS )
                {
                  printk("%s-timesync_ioctl: \
                      ixTimeSyncAccAuxTargetTimePoll failed\n",
                          DRIVERNAME);
                  ret = -EINVAL;
                }
        }
        else if ( cmd == IOCTL_AUX_TIME_POLL )
        {
                if ( ixTimeSyncAccAuxTimePoll(pTimePollIoctl->auxMode,
                              &pTimePollIoctl->PollFlag,
                              &pTimePollIoctl->timeVal)
                      != ICP_TIMESYNCACC_SUCCESS )
                {
                  printk("%s-timesync_ioctl: \
                      ixTimeSyncAccAuxTimePoll failed\n",
                          DRIVERNAME);
                  ret = -EINVAL;
                }
        }
        break;

      case IOCTL_RESET:

            /*
            *
            * request a complete hardware reset
            *

            * init events *!!!!!!!!  verify
            */

            for (i=0; i < NUM_EVENTS; i++)
            {
                init_waitqueue_head(&g_notify_event[i]);
                g_eventflag[i] = 0;
            }

        /* reset the 1588 hardware*/
            if ( ixTimeSyncAccReset() != ICP_TIMESYNCACC_SUCCESS )
            {
                printk("%s-timesync_ioctl: \
                    ixTimeSyncAccReset failed\n",
                        DRIVERNAME);
                ret = -EINVAL;
            }
            break;

      case IOCTL_CHNL_RESET:

            /*
             *
             * request to reset a single channel
             *
             * copy the passed in parameter (port number to reset)
             */

        memcpy(&ptpPort, buff, sizeof(IxTimeSyncAcc1588PTPPort));

        /* reset the specified channel */
            if ( ixTimeSyncAccChnlReset(ptpPort) != ICP_TIMESYNCACC_SUCCESS )
            {
                printk("%s-timesync_ioctl: \
                    ixTimeSyncAccChnlReset failed\n",
                        DRIVERNAME);
                ret = -EINVAL;
            }
            break;

      case IOCTL_SHOW_ALL:

            if ( ixTimeSyncAccShow() != ICP_TIMESYNCACC_SUCCESS )
            {
                printk("%s-timesync_ioctl: \
                    ixTimeSyncAccShow failed\n",
                        DRIVERNAME);
                ret = -EINVAL;
            }
            break;

      case IOCTL_STATS_GET:

            if ( ixTimeSyncAccStatsGet((IxTimeSyncAccStats*)buff)
                  != ICP_TIMESYNCACC_SUCCESS )
            {
                printk("%s-timesync_ioctl: \
                    ixTimeSyncAccStatsGet failed\n",
                      DRIVERNAME);
                ret = -EINVAL;
            }
        break;

      case IOCTL_PORT_VERSION_SET:

        pVersionIoctl = (IxTimeSyncAccVersionIoctl*)buff;

            if ( ixTimeSyncAccPTPVersionSet(pVersionIoctl->ptpPort,
                               pVersionIoctl->ptpVersion)
                    != ICP_TIMESYNCACC_SUCCESS )
            {
                printk("%s-timesync_ioctl: \
                    ixTimeSyncAccPTPVersionSet failed\n",
                        DRIVERNAME);
                ret = -EINVAL;
            }
            break;

      case IOCTL_PORT_VERSION_GET:

            pVersionIoctl = (IxTimeSyncAccVersionIoctl*)buff;

            if ( ixTimeSyncAccPTPVersionGet(pVersionIoctl->ptpPort,
                               &pVersionIoctl->ptpVersion)
                    != ICP_TIMESYNCACC_SUCCESS )
            {
                printk("%s-timesync_ioctl: \
                    ixTimeSyncAccPTPVersionGet failed\n",
                        DRIVERNAME);
                ret = -EINVAL;
            }
        break;


      case IOCTL_PORT_OPERATION_MODE_SET:

        pOperationModeIoctl = (IxTimeSyncAccOperationModeIoctl*)buff;

            if ( ixTimeSyncAccPTPOperationModeSet(pOperationModeIoctl->ptpPort,
                               pOperationModeIoctl->ptpOpMode)
                    != ICP_TIMESYNCACC_SUCCESS )
            {
                printk("%s-timesync_ioctl: \
                    ixTimeSyncAccPTPOperationModeSet failed\n",
                        DRIVERNAME);
                ret = -EINVAL;
            }
            break;

      case IOCTL_PORT_OPERATION_MODE_GET:

            pOperationModeIoctl = (IxTimeSyncAccOperationModeIoctl*)buff;

            if ( ixTimeSyncAccPTPOperationModeGet(pOperationModeIoctl->ptpPort,
                               &pOperationModeIoctl->ptpOpMode)
                    != ICP_TIMESYNCACC_SUCCESS )
            {
                printk("%s-timesync_ioctl: \
                    ixTimeSyncAccPTPOperationModeGet failed\n",
                        DRIVERNAME);
                ret = -EINVAL;
            }
        break;


      case IOCTL_STATS_RESET:

            ixTimeSyncAccStatsReset();
            break;

        default:
            printk("%s-timesync_ioctl: unknown command (%d)\n",
                DRIVERNAME, cmd);
        return -EINVAL;
    }


    /* if data is being read from the driver*/
    if ( _IOC_DIR(cmd) & _IOC_READ )
    {
        if ( copy_to_user((void*)arg, buff, buffsize) )
        {
            printk("%s-timesync_ioctl: could not copy data to user space.\n",
                DRIVERNAME);
            return -EFAULT;
        }
    }

    return ret;
}

/*****************************************************************************
  Description:
    TargTimeCallback is a callback function that will be called from the HAL
    when the target time has expired and generates an interrupt

  Parameters:
    targetTime   target time register timestamp

  Returns:
      none

******************************************************************************/
void TargTimeCallback(IxTimeSyncAccTimeValue targetTime)
{
    /* copy the target time value to the global value to be read by the
    * notify ioctl
    */
    memcpy(&gTargTimeIoctl, &targetTime, sizeof(IxTimeSyncAccTimeValue));

    g_eventflag[TARG_TIME_EVENT_NUM] = 1;

    /*
    signal the notify ioctl that the target time has expired
    */
    wake_up_interruptible(&g_notify_event[TARG_TIME_EVENT_NUM]);
}

/*****************************************************************************
  Description:
    AuxTimeCallback is a callback function that will be called from the HAL
    when an aux time interrupt has occurred

  Parameters:
    auxMode   master, slave, or any
    auxTime   aux time register timestamp

  Returns:
      none

******************************************************************************/
void AuxTimeCallback(IxTimeSyncAccAuxMode auxMode,
         IxTimeSyncAccTimeValue auxTime)
{
    /*
     * copy the aux time value and aux mode to the global value
     * to be read by the notify ioctl
     */

    gAuxTimeIoctl.auxMode = auxMode;
    memcpy(&gAuxTimeIoctl.auxTime, &auxTime, sizeof(IxTimeSyncAccAuxTimeIoctl));

    g_eventflag[AUX_TIME_EVENT_NUM] = 1;

    /*
     * signal the notify ioctl that the aux timestamp has been set
     */
    wake_up_interruptible(&g_notify_event[AUX_TIME_EVENT_NUM]);
}

/*****************************************************************************
  Description:
    AuxTargetTimeCallback is a callback function that will be called from the HAL
    when the aux target time has expired and generates an interrupt

  Parameters:
    auxTargetTime   aux target time register timestamp

  Returns:
      none

******************************************************************************/
void AuxTargetTimeCallback(IxTimeSyncAccTimeValue auxTargetTime)
{
    memcpy(&gAuxTargTimeIoctl, &auxTargetTime, sizeof(IxTimeSyncAccTimeValue));

    g_eventflag[AUX_TARG_TIME_EVENT_NUM] = 1;

    wake_up_interruptible(&g_notify_event[AUX_TARG_TIME_EVENT_NUM]);
}

/*****************************************************************************
  Description:
    PulsePerSecondCallback is a callback function that will be called from the HAL
    when the pulse per second time has expired and generates an interrupt

  Parameters:
    pulsePerSecond   pulse per second register timestamp

  Returns:
      none

******************************************************************************/
void PulsePerSecondCallback(UINT32 pulsePerSecond)
{
    gPulsePerSecondIoctl = pulsePerSecond;

    g_eventflag[PPS_EVENT_NUM] = 1;

    /* signal the notify ioctl that the pulse per second time has expired*/

    wake_up_interruptible(&g_notify_event[PPS_EVENT_NUM]);
}



module_init(timesync_init);
module_exit(timesync_close);



