/*****************************************************************************
 * @ingroup 1588
 *
 * @file 1588.h
 *
 * @description
 *		The 1588 driver module enables a client application to have
 * 		access to the 1588 time synchronization hardware assist block. 
 * 		A client application can use this driver to retreive tx and rx timestamps 
 * 		of IEEE 1588(TM) PTP messages captured on ethernet or CAN ports. 
 * 		Access is provided by a set of APIs that can be called directly or through 
 * 		ioctl commands.  
 *
 * This file is provided under a dual BSD/GPLv2 license.  When using or 
 *   redistributing this file, you may do so under either license.
 * 
 *   GPL LICENSE SUMMARY
 * 
 *   Copyright(c) 2007,2008 Intel Corporation. All rights reserved.
 * 
 *   This program is free software; you can redistribute it and/or modify 
 *   it under the terms of version 2 of the GNU General Public License as
 *   published by the Free Software Foundation.
 * 
 *   This program is distributed in the hope that it will be useful, but 
 *   WITHOUT ANY WARRANTY; without even the implied warranty of 
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 *   General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License 
 *   along with this program; if not, write to the Free Software 
 *   Foundation, Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *   The full GNU General Public License is included in this distribution 
 *   in the file called LICENSE.GPL.
 * 
 *   Contact Information:
 *   Intel Corporation
 * 
 *   BSD LICENSE 
 * 
 *   Copyright(c) 2007,2008 Intel Corporation. All rights reserved.
 *   All rights reserved.
 * 
 *   Redistribution and use in source and binary forms, with or without 
 *   modification, are permitted provided that the following conditions 
 *   are met:
 * 
 *     * Redistributions of source code must retain the above copyright 
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright 
 *       notice, this list of conditions and the following disclaimer in 
 *       the documentation and/or other materials provided with the 
 *       distribution.
 *     * Neither the name of Intel Corporation nor the names of its 
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 * 
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 *   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 *   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 *   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * 
 *  version: Embedded.L.1.0.34
 ****************************************************************************/


#ifndef TS1588_H
#define TS1588_H

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/socket.h>
#include <linux/poll.h>   
#include <linux/init.h>
#include <linux/fs.h>     
#include <linux/errno.h>    
#include <linux/types.h>    
#include <linux/proc_fs.h>
#include <linux/fcntl.h>    
#include <linux/interrupt.h>
#include <asm/system.h>     
#include <asm/uaccess.h>
#include <linux/pci.h>
#include <linux/cdev.h>

#include "ixtimesyncacc.h"
#include "linux_ioctls.h"

#define DRIVERNAME "1588"

#define VENDOR_ID 0x8086
#define DEVICE_ID 0x503c
#define IO_MEM_BAR 0


// enumeration of events used
typedef enum _NotifyEvent
{
    TARG_TIME_EVENT_NUM,
    AUX_TIME_EVENT_NUM,
    AUX_TARG_TIME_EVENT_NUM,
    PPS_EVENT_NUM,
    NUM_EVENTS
} NotifyEvent;

// Linux id structure
static struct pci_device_id pci_ids[] = {
	{ PCI_DEVICE(VENDOR_ID, DEVICE_ID) },
	{ 0, },
};
MODULE_DEVICE_TABLE(pci, pci_ids);

// Linux functions
int timesync_open(struct inode *inode, struct file *filp);
int timesync_release(struct inode *inode, struct file *filp);
int timesync_ioctl(struct inode *inode, struct file *filp, unsigned int cmd,
				unsigned long arg);
void timesync_close(void);
int pci_suspend(struct pci_dev *dev, pm_message_t state);
int pci_resume(struct pci_dev *dev);
int pci_probe(struct pci_dev *dev, const struct pci_device_id *id);
void pci_remove(struct pci_dev *dev);
irqreturn_t timesync_isr(int irq, void *dev_id, struct pt_regs *regs);

// private functions
int save_reg_state(void);
int restore_reg_state(void);
int disable_interrupts(void);
int restore_interrupts(void);

// Linux file operations
struct file_operations file_ops = 
{
	.owner		= THIS_MODULE,
	.open     = timesync_open,
	.release    = timesync_release,	
	.ioctl		= timesync_ioctl,
};

// Linux pci operations
static struct pci_driver pci_ops = {
	.name = DRIVERNAME,
	.id_table  = pci_ids,
	.probe     = pci_probe,
	.remove    = pci_remove,
	.suspend   = pci_suspend,
	.resume    = pci_resume,
};

// 1588 register states
typedef struct reg_states
{
	unsigned int AmmsInterrupt;
	unsigned int AsmsInterrupt;
	unsigned int TtmInterrupt;
	unsigned int AtmInterrupt;
	unsigned int PpsmInterrupt;
	IxTimeSyncAccTimeValue SystemTime;
	IxTimeSyncAccTimeValue TargetTime;
	IxTimeSyncAccTimeValue AuxTargetTime;
	unsigned int PulsePerSecond;
	unsigned int Addend;
	unsigned int MasterMode[ICP_TIMESYNCACC_1588PORT_INVALID];
	unsigned int MsgTimestamp[ICP_TIMESYNCACC_1588PORT_INVALID];
} reg_states_t;

// private driver data
typedef struct drvr_data
{
	dev_t devnum;
	struct cdev cdev;
	void *mem_virt;
	unsigned int mem_base;
	unsigned int mem_size;
    unsigned int irq;
    unsigned int suspend;
    reg_states_t state;

} drvr_data_t;

// driver data struct
drvr_data_t g_drvr_data;

// Global event variables
unsigned int g_eventflag[NUM_EVENTS];
wait_queue_head_t g_notify_event[NUM_EVENTS];

// ioctl call data structures for notify events
IxTimeSyncAccTimeValue gTargTimeIoctl;
IxTimeSyncAccTimeValue gAuxTargTimeIoctl;
IxTimeSyncAccAuxTimeIoctl gAuxTimeIoctl;
unsigned int gPulsePerSecondIoctl;

// interrupt event flags
UINT32 ixTimeSyncAccEventAmmsFlagGet(void);
UINT32 ixTimeSyncAccEventAsmsFlagGet(void);
UINT32 ixTimeSyncAccEventAtmFlagGet(void);
UINT32 ixTimeSyncAccEventPpsmFlagGet(void);
UINT32 ixTimeSyncAccEventTtmFlagGet(void);


// HAL callback function pointers
IxTimeSyncAccTargetTimeCallback TargetTimeCallbackPtr;
IxTimeSyncAccAuxTimeCallback AuxTimeCallbackPtr;
IxTimeSyncAccAuxTargetTimeCallback AuxTargetTimeCallbackPtr;
IxTimeSyncAccPulsePerSecondCallback PulsePerSecondCallbackPtr;

// HAL callback function prototypes
void TargTimeCallback(IxTimeSyncAccTimeValue targetTime);
void AuxTimeCallback(IxTimeSyncAccAuxMode auxMode, IxTimeSyncAccTimeValue auxTime);
void AuxTargetTimeCallback(IxTimeSyncAccTimeValue auxTargetTime);
void PulsePerSecondCallback(UINT32 pulsePerSecond);

#endif
