/*****************************************************************************
 * This file is provided under a dual BSD/GPLv2 license.  When using or 
 *   redistributing this file, you may do so under either license.
 * 
 *   GPL LICENSE SUMMARY
 * 
 *   Copyright(c) 2007,2008 Intel Corporation. All rights reserved.
 * 
 *   This program is free software; you can redistribute it and/or modify 
 *   it under the terms of version 2 of the GNU General Public License as
 *   published by the Free Software Foundation.
 * 
 *   This program is distributed in the hope that it will be useful, but 
 *   WITHOUT ANY WARRANTY; without even the implied warranty of 
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 *   General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License 
 *   along with this program; if not, write to the Free Software 
 *   Foundation, Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *   The full GNU General Public License is included in this distribution 
 *   in the file called LICENSE.GPL.
 * 
 *   Contact Information:
 *   Intel Corporation
 * 
 *   BSD LICENSE 
 * 
 *   Copyright(c) 2007,2008 Intel Corporation. All rights reserved.
 *   All rights reserved.
 * 
 *   Redistribution and use in source and binary forms, with or without 
 *   modification, are permitted provided that the following conditions 
 *   are met:
 * 
 *     * Redistributions of source code must retain the above copyright 
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright 
 *       notice, this list of conditions and the following disclaimer in 
 *       the documentation and/or other materials provided with the 
 *       distribution.
 *     * Neither the name of Intel Corporation nor the names of its 
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 * 
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 *   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 *   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 *   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * 
 *  version: Embedded.L.1.0.34
 *****************************************************************************/

/**
 ******************************************************************************
 * @ingroup EDMA_CLIENT_API An API used by client drivers
 *          to access the EDMA driver.
 *      File:  dma_api.h
 *
 * @description
 * This group of functions comprise the core functionality of the Enhanced 
 * DMA driver and manage the data structures internal to the driver.
 *****************************************************************************/

#ifndef DMA_API_H
#define DMA_API_H


#include "dma_internals.h"


/***************** NON-IOAT ROUTINES  ********************/
/* Internal */
struct edma_client_internal *edma_client_alloc(void);
void edma_client_release(struct edma_client *client);
struct edma_chan * edma_chan_alloc(void);

/* Prototypes - Driver-to-Driver Level */
/**
 ***************************************************************************** 
 * @ingroup EDMA_CLIENT_API
 * Releases a channel from association with a particular client.
 * @description
 * @param pointer to a edma_chan struct allocated with edma_chan_alloc.
 * @retval NOTHING
 * @reentrant - Yes
 * @context - Non-ISR
 *****************************************************************************/
void edma_chan_release(struct edma_chan * chan);

/**
 ***************************************************************************** 
 * @ingroup EDMA_CLIENT_API
 * Adds a client struct to an internal master client list.
 * @description
 * The driver does book-keeping to keep track of clients that have requested
 * access through the API.
 * @param ptr to edma_client struct created with edma_client_alloc
 * @retval SUCCESS (0)
 * @retval FAILED (-1)
 * @reentrant - Yes
 * @context - Non-ISR
 *****************************************************************************/
struct edma_client * edma_client_register(edma_callback_t callback);

/**
 ***************************************************************************** 
 * @ingroup EDMA_CLIENT_API
 * Removes a client from the internal
 * master client list.
 * @description
 * Use this function before calling the edma_client_release function.
 * @param edma_client created with edma_client_alloc
 * @retval NOTHING
 * @reentrant - Yes
 * @context - Non-ISR
 *****************************************************************************/
void edma_client_unregister(struct edma_client *client);

/**
 ***************************************************************************** 
 * @ingroup EDMA_CLIENT_API
 * Returns the contents of the FERR for a specified cookie.
 * @description
 * EXAMPLE: a return value of 0x20014 indicates the problematic descriptor
 * was the second descriptor in the chain and the FERR value was 0x0014.
 * @param channel - ptr to a channel allocated by edma_chan_alloc
 * @param cookie  - integer cookie value returned by edma_memcpy
 * @retval 0 = no error information was found for the particular cookie or
 * a positive integer containing the FERR error code (1st two bytes) and the
 * number of the problematic descriptor.
 * @reentrant - Yes
 * @context - Non-ISR
 *****************************************************************************/
uint32_t edma_get_errors(struct edma_chan * chan, edma_cookie_t cookie);

/**
 ***************************************************************************** 
 * @ingroup EDMA_CLIENT_API
 * Changes the channel arbitration scheme from round robin to 50% bandwidth
 * for a given channel.
 * 
 * @description
 * Integer values 0-3 are valid channel numbers. Any other value will
 * reset the arbitration scheme to the default round robin.
 * 
 * @param channel - a valid channel number. See description above.
 * @retval EDMA_CP_FAIL = -1 Failed to reset channel priority
 * @retval EDMA_CP_DEFAULT 0 Set channel priority to default (round robin)
 * @retval EDMA_CP_VALID_CHANNEL 1 Set user specified channel to
 * have top priority
 * @reentrant - Yes
 * @context - Non-ISR
 *****************************************************************************/
int32_t edma_change_channel_priority(uint32_t channel);

/**
 ***************************************************************************** 
 * @ingroup EDMA_CLIENT_API
 * Polling function to determine if a transaction has successfully completed.
 * @description
 * Use this function for polling the status of a particular
 * transaction if the client has no callback function. Note, there
 * are 16 descriptors, so it is possible for cookie information to
 * expire.
 * @param chan - channel allocated by edma_chan_alloc.
 * @param cookie - integer cookie value returned by edma_memcpy.
 * @retval EDMA_SUCCESS (0) (successfully completed)
 * @retval EDMA_IN_PROGRESS (1)
 * @retval EDMA_ERROR (2) (error occurred)
 * @retval EDMA_NO_RECORD (-1) (cookie expired no record)
 * @reentrant - Yes
 * @context - Non-ISR
 *****************************************************************************/
uint32_t edma_is_complete(struct edma_chan * chan, edma_cookie_t cookie);

/**
 ***************************************************************************** 
 * @ingroup EDMA_CLIENT_API
 * Used by client drivers to request channel(s)
 * @description
 * @param pointer to a registered edma_client struct allocated with
 * edma_client_register.
 * @param number of channels requested per client.
 * @retval NOTHING
 * @reentrant - Yes
 * @context - Non-ISR
 *****************************************************************************/
void edma_chan_request(struct edma_client * client, uint32_t channels);

/**
 ***************************************************************************** 
 * @ingroup EDMA_CLIENT_API
 * Performs the memory transfer operations (Mem-to-Mem).
 * @description
 * @param channel (0-3)
 * @param physical address of the memory destination
 * @param physical address of the memory source
 * @param entire transfer size in bytes
 * @retval cookie value greater than 0 = valid cookie
 * @retval cookie value less than or equal to 0 = a problem occurred
 * @reentrant - Yes
 * @context - Non-ISR
 *****************************************************************************/
edma_cookie_t edma_memcpy(uint32_t memory_ops, struct edma_chan *chan, 
edma_addr_t dest, edma_addr_t src, size_t size);


/**
 ***************************************************************************** 
 * @ingroup EDMA_CLIENT_API
 * Sets the memory transfer type.
 * @param client: pointer to a client struct
 * @param callback a callback function ptr
 * @retval NOTHING
 * @reentrant - Yes
 * @context - Non-ISR
 *****************************************************************************/
void edma_set_completion_callback(struct edma_client * client,
edma_completion_callback_t callback);

/* Set tranfer mode */
/**
 ***************************************************************************** 
 * @ingroup EDMA_CLIENT_API
 * Sets the memory transfer type.
 * @param channel (0-3)
 * @param memory_ops: type of memory tranfers (e.g. Memory to Memory)
 * @retval value  0 = set transaction type successfully
 * @retval value less than < 0  = transaction type not set
 * @reentrant - Yes
 * @context - Non-ISR
 *****************************************************************************/
int32_t edma_set_transaction(struct edma_chan *chan, uint32_t memory_ops);

/**
 *****************************************************************************
 * @ingroup EDMA_CLIENT_API
 * An Export Table for the Client API
 * @description
 *   This structure is intended as a table for intermodule communication 
 *   (Linux only). It holds function pointers to all of the client API 
 *   functions. The client declares a variable of type edma_api_export and 
 *   'fills' the table with edma_get_functions, eliminating successive calls
 *   to Linux's symbol getting macro.
 *****************************************************************************/
struct edma_api_export {
    struct edma_client * (*edma_client_register)
    (edma_callback_t callback); /**<ptr to edma_client_register */ 
	
    void (*edma_client_unregister)(struct edma_client *); /**<ptr to
                                                     edma_client_unregister*/ 
                                                     	
    void (*edma_chan_request)(struct edma_client *, uint32_t); /**<ptr to
						       edma_chan_alloc */
												
    void (*edma_chan_release)(struct edma_chan *); /**<ptr to
						       edma_chan_release */
												    
    edma_cookie_t (*edma_memcpy)(uint32_t,
    struct edma_chan *, edma_addr_t,
    edma_addr_t, size_t); /**<ptr to edma_memcpy >**/

    uint32_t (*edma_is_complete) (struct edma_chan *,
    edma_cookie_t); /**<ptr to edma_is_complete >**/

    uint32_t (*edma_get_errors)(struct edma_chan *,
    edma_cookie_t); /**<ptr to edma_get_errors >**/
    
    void (*edma_set_completion_callback)
    (struct edma_client *, edma_completion_callback_t);/**< ptr to 
    edma_set_completion callback >**/
    
};

/* INTERNAL */
/**
 ***************************************************************************** 
 * @ingroup EDMA_CLIENT_API
 * Fill function table (function ptrs to the client API).
 * @description
 * @param api pointer to a edma_api_export structure
 * @retval SUCCESS
 * @retval FAILED
 * @reentrant - Yes
 * @context - Non-ISR
 *****************************************************************************/
int32_t edma_get_functions(struct edma_api_export * api);


/**
 *****************************************************************************
 * @ingroup EDMA_CLIENT_API
 * A type definition  for edma_get_functions. 
 * @description
 *****************************************************************************/
typedef int32_t (*edma_get_functions_t) (struct edma_api_export *);

/* INTERNAL */
struct edma_descriptor_internal * get_descriptor_lists_ptr(int);

 
#endif

