/*****************************************************************************
 * This file is provided under a dual BSD/GPLv2 license.  When using or 
 *   redistributing this file, you may do so under either license.
 * 
 *   GPL LICENSE SUMMARY
 * 
 *   Copyright(c) 2007,2008 Intel Corporation. All rights reserved.
 * 
 *   This program is free software; you can redistribute it and/or modify 
 *   it under the terms of version 2 of the GNU General Public License as
 *   published by the Free Software Foundation.
 * 
 *   This program is distributed in the hope that it will be useful, but 
 *   WITHOUT ANY WARRANTY; without even the implied warranty of 
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 *   General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License 
 *   along with this program; if not, write to the Free Software 
 *   Foundation, Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *   The full GNU General Public License is included in this distribution 
 *   in the file called LICENSE.GPL.
 * 
 *   Contact Information:
 *   Intel Corporation
 * 
 *   BSD LICENSE 
 * 
 *   Copyright(c) 2007,2008 Intel Corporation. All rights reserved.
 *   All rights reserved.
 * 
 *   Redistribution and use in source and binary forms, with or without 
 *   modification, are permitted provided that the following conditions 
 *   are met:
 * 
 *     * Redistributions of source code must retain the above copyright 
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright 
 *       notice, this list of conditions and the following disclaimer in 
 *       the documentation and/or other materials provided with the 
 *       distribution.
 *     * Neither the name of Intel Corporation nor the names of its 
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 * 
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 *   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 *   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 *   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * 
 *  version: Embedded.L.1.0.34
 *****************************************************************************/
/**
 *****************************************************************************
 * @defgroup EDMA_CLIENT_API  API for Clients (Drivers or Applications)
 * File:  dma_client_api.c
 * $Revision:1.0$
 * @purpose
 * This group of APIs is to be used by the client drivers
 * to access enhanced DMA.
 * IMPORTANT NOTE: 
 * In the functions below the reentrant tag refers to the SMP concept.
 * The context tag for all of the functions below is Non-ISR, indicating 
 * that the function should not be called from a client's ISR.
 *****************************************************************************/
 
/**
 ******************************************************************************
 * @ingroup EDMA_CLIENT_API
 *
 * @file dma_client_api.c
 *
 * @description
 * This group of functions comprises the core functionality of Enhanced 
 * DMA driver and manage the data structures internal to the driver.
 *****************************************************************************/
 
/*****************************************************************************
 *
 * Contents:
 *        Definitions of following exported data:
 *            
 *        Definitions of following exported functions
 *            edma_client_alloc
 *            edma_client_release
 *            edma_client_register
 *            edma_client_unregister
 *            edma_chan_alloc
 *            edma_chan_release
 *            edma_memcpy
 *            edma_is_complete
 *
 *        Definitions of following local functions:
 *            release_client_resources
 *            check_channel_counts
 *****************************************************************************/




#ifdef __FreeBSD__
 
#include <sys/types.h> /* Data types */
#include <sys/systm.h>
#include <machine/bus.h>
#include <sys/bus.h>

#endif

#include "os.h"
#include "os_list.h"
#include "dma_api.h"


static struct edma_api_export edma_api = {
    .edma_client_register = edma_client_register,
    .edma_client_unregister = edma_client_unregister,
    .edma_chan_request = edma_chan_request,
    .edma_chan_release = edma_chan_release,
    .edma_memcpy = edma_memcpy,
    .edma_is_complete = edma_is_complete,
    .edma_get_errors = edma_get_errors,
    .edma_set_completion_callback = edma_set_completion_callback
};
/* Initialized elsewhere... */
extern struct edma_descriptor_list g_descriptor_lists[MAX_NO_CHANS];
extern struct edma_chan * g_edma_channel_list[MAX_NO_CHANS];
struct os_list_start g_edma_client_list     = {NULL, NULL};
extern os_spinlock_t g_channel_lock;

#ifdef __FreeBSD__
extern bus_dma_tag_t bus_tag;
#endif

uint32_t g_channel_ccr_offsets[] = { EDMA_CHAN0_CCR,
EDMA_CHAN1_CCR, EDMA_CHAN2_CCR, EDMA_CHAN3_CCR};

void release_client_resources(struct edma_client_internal *client_internal);
void check_channel_counts(void);




/**
 ***************************************************************************** 
 *****************************************************************************/
void edma_set_completion_callback(struct edma_client * client,
edma_completion_callback_t callback)
{
    client->edma_completion_callback = callback;
}




/***************************************************************************** 
 * Abstract
 * Frees the resource structures associated with a client
 * Side Effects:
 * Assumptions:
 * The edma_client_internal structure is not to be confused with edma_client.
 * This function is not part of the client driver API or the application
 * interface.
 * It is called internally.
 * Used internally only. 
 * Return Values: Nothing
 *****************************************************************************/
void release_client_resources(struct edma_client_internal *client_internal
                              /* internal client */)
{
    struct edma_client * client = NULL;
    struct edma_chan * chan = NULL;
    uint32_t i = 0;
    struct edma_descriptor_block  * block = NULL;
    int32_t size = 
    (sizeof(struct edma_descriptor_internal)  * DEFAULT_DESCRIPTOR_NO);
    
    if(client_internal != NULL)
    {
        client = &(client_internal->pub);

        /* Remove each channel from the list of channels...*/
        while(i < MAX_NO_CHANS)
        {
           
            if(client->channels[i] == IN_USE)
            {
                client->channels[i] = NOT_IN_USE;
                edma_chan_release(g_edma_channel_list[i]);
                DMA_DEBUG_PRINT_OS_1("RELEASING CHANNEL %i\n", i);

                /* Send out a message to client. */
                if(client->edma_callback != NULL)
                {
                    client->edma_callback(client, 
                    chan, EDMA_RESOURCE_REMOVED);
                }
                   
                /* 
                 * This loop assumes no transactions are 
                 * outstanding. 
                 */ 
                do
                {
                     block = remove_block(i);    
                     if(block != NULL)
                     {
                         DMA_DEBUG_PRINT_OS_1("REMOVING BLOCK %x\n",
                         (int)block); 
                         #ifdef __FreeBSD__
                         os_free_coherent((void *)bus_tag,
                         size, block->pool_addr, 
                         &block->handle);
                         #endif
                         #ifdef __KERNEL__
                         os_free_coherent(NULL, size, 
                         block->pool_addr, &(block->handle));
                         #endif
                    
                         os_free(block, STANDARD_MEM);
                     }
                 }while(block != NULL);

           } /* IF */

           i++;
        } /* WHILE */
        
        

    } /* IF */
}

/***************************************************************************** 
 * Abstract
 * Obtains more available channels for a client if they are free. Walks entire
 * list of clients re-allocating resources based on 
 * chan_count v. chan_desired.
 * Side Effects:
 * Assumptions:
 * There is no channel balancing. This function does not arbitrate channel
 * use. If a client decides to monopolize channels, it is possible for a 
 * client to obtain all 4 channels.
 *
 * The edma_client_internal structure is not to be confused with edma_client.
 * This function is not part of the client driver API or the application
 * interface.
 * It is called internally.
 * Used internally only. 
 * Return Values: Nothing
 *****************************************************************************/
void check_channel_counts(void)
{
    struct edma_client_internal * client = NULL;
    int i; 
    client = (struct edma_client_internal *) g_edma_client_list.next_ptr;
    while(client != NULL)
    {
        os_spin_lock(&g_channel_lock);
        for(i = 0; i < MAX_NO_CHANS; i++)
        {
            if(client->pub.chan_desired > client->pub.chan_count)
            {
               if(g_edma_channel_list[i]->sw_in_use == NOT_IN_USE)
               {
                   client->pub.channels[i] = IN_USE;
                   g_edma_channel_list[i]->sw_in_use = IN_USE;
                   client->pub.chan_count++;
                   
                   if(client->pub.edma_callback != NULL)
                   {
                       client->pub.edma_callback(&(client->pub), 
                       g_edma_channel_list[i], EDMA_RESOURCE_ADDED);
                   }
               }

            } 
        
        } 
        os_spin_unlock(&g_channel_lock); 
        client = (struct edma_client_internal *) client->next_ptr;  
    }
    
}
/**
 ***************************************************************************** 
 * @ingroup EDMA_CLIENT_API
 * Allocates a client
 * @description
 * Creates a new client structure, along with additional allocations
 * internal to the driver.  Do not pass client structures to the
 * register function that have not been allocated with edma_client_alloc.
 * Similarly, only use edma_client_release to free the structure.
 * @param NOTHING
 * @retval NON-NULL ptr to a edma_client struct
 * @retval NULL - unable to allocate client struct
 * @reentrant - Yes
 * @context - Non-ISR
 *****************************************************************************/
struct edma_client_internal * edma_client_alloc()
{
    struct edma_client_internal *client = NULL;
    
    /* Create the internal structure. */
    client = os_malloc(sizeof(struct edma_client_internal), 
    STANDARD_MEM, GENERAL_KERNEL);
    

    if(client == NULL)
    {
        DMA_DEBUG_PRINT_OS_0(ERROR_CLIENT_ALLOC);
 
        return NULL;
    }
    /* Init the region of mem with 0s. */
    memset(client, 0, sizeof(*client));
    

    return client;
}



/**
 ***************************************************************************** 
 * @ingroup EDMA_CLIENT_API
 * Releases client structure.
 * @description
 * Use this function after unregistering the client.
 * Releases the client structure, along with the additional internal
 * allocations made in edma_client_alloc.
 * @param client - pointer to driver-created client
 * @retval NOTHING
 * @reentrant - Yes
 * @context - Non-ISR
 *****************************************************************************/
void edma_client_release(struct edma_client *client)
{
    struct edma_client_internal *client_internal = NULL;
    /*
        This operation is very dependent upon the
        location of the client in the client_internal
        structure.

        In Linux 2.6.x kernels, there is a container_of
        macro that returns the pointer of the container struct
        given a member of that struct, which is the another way
        of handling the structure-in-structure issue.
    */
   
    if(client != NULL)
    {
        client_internal = get_client_internal(client);
        os_free(client_internal, STANDARD_MEM);
    }

}




/**
 ***************************************************************************** 
 *****************************************************************************/
struct edma_client * edma_client_register(edma_callback_t callback)
{
    struct edma_client_internal *client_internal = edma_client_alloc();
   
    
   
    if(client_internal != NULL)
    {
         
        client_internal->pub.edma_callback = callback;
        
        os_list_insert_element_at_end((struct os_list_start*)client_internal,
        &g_edma_client_list);
        client_internal->pub.global_node = g_edma_client_list;
        
    }
    
    return &(client_internal->pub);

}



/**
 ***************************************************************************** 
 *****************************************************************************/
void edma_client_unregister(struct edma_client *client)
{
    struct edma_client_internal *client_internal = NULL;
    
    
    
    if(client != NULL)
    {
        client_internal = get_client_internal(client);
        os_list_remove_element((struct os_list_start *)client_internal,
        &g_edma_client_list);    
        /*Release the channels associated with the client */
        release_client_resources(client_internal);
        
        /* Free the client_internal... */
        edma_client_release(client);
        
        check_channel_counts();
        
    }
    
}




/**
 ***************************************************************************** 
 *****************************************************************************/
void edma_chan_release(struct edma_chan * chan)
{
    os_spin_lock(&g_channel_lock);
    chan->sw_in_use = NOT_IN_USE;
    chan->memory_ops = 0;
    
    chan->client = NULL;
    os_spin_unlock(&g_channel_lock);
    DMA_DEBUG_PRINT_OS_1("RELEASED CHANNEL NUMBER %i\n", chan->channel_no);
}

/**
 ***************************************************************************** 
 *****************************************************************************/
void edma_chan_request(struct edma_client * client, uint32_t channels)
{
    struct edma_chan *chan = NULL;
    int i = 0;


    /* Make sure the number of channels requested is within */
    if(channels > MAX_NO_CHANS)
    {
        client->chan_desired = MAX_NO_CHANS;
    }
    else
    {
        client->chan_desired = channels;
    }

    for(i = 0; i < client->chan_desired; i++)
    {

        
        chan = edma_chan_alloc();
        
        
        if(chan !=NULL)
        {
            if(client->edma_callback != NULL)
            {
               
               client->edma_callback(client, chan, EDMA_RESOURCE_ADDED);

               /* Add channel to client's channel list */
               client->channels[chan->channel_no] = IN_USE;
               chan->client = client;

            }
            client->chan_count++;   
        }
        chan = NULL;
    }
}
/**
 ***************************************************************************** 
 * @ingroup EDMA_CLIENT_API
 * Allocates a single channel from an available list.
 * @description
 * Called by edma_chan_request. 
 * @retval NON-NULL -ptr to a edma_chan struct
 * @retval NULL - unable to allocate
 * @reentrant - Yes
 * @context - Non-ISR
 *****************************************************************************/
struct edma_chan * edma_chan_alloc()
{
    struct edma_chan *chan = NULL;
    int i = 0;
    os_spin_lock(&g_channel_lock);
    
    for(i = 0; i < MAX_NO_CHANS; i++)
    {
       
        /* Search for a channel requested. If available, 
        return a valid channel. */
        
        if(g_edma_channel_list[i]->sw_in_use == NOT_IN_USE)
        {
             chan = g_edma_channel_list[i];
             g_edma_channel_list[i]->sw_in_use = IN_USE;
             break;      
            
        }
       
        }
     os_spin_unlock(&g_channel_lock);

    if(chan == NULL)
    {
        DMA_DEBUG_PRINT_OS_1("UNABLE TO OBTAIN CHANNEL %i\n", i);
    }

    return chan;
}




/**
 ***************************************************************************** 
 * @ingroup EDMA_CLIENT_API
 * Returns function ptrs to the API.
 * @description 
 * This function fills a function table with pointers to functions of the 
 * client API eliminating successive calls to the symbol request mechanism
 * in Linux.
 * @param ptr to an edma_api_export struct
 * @retval SUCCESS (0)
 * @retval FAILED (-1)
 * @reentrant - Yes
 * @context - Non-ISR
 *****************************************************************************/
int32_t edma_get_functions(struct edma_api_export * funcs)
{
    int32_t err = SUCCESS;
    if(funcs != NULL)
    {
        *funcs = edma_api;
    }
    else
    {
        DMA_DEBUG_PRINT_OS_0("BAD INITIALIZATION OF FUNCTION TABLE\n");
        err = FAILED;
    }
    return err;
}
/**
 ***************************************************************************** 
 *****************************************************************************/

uint32_t edma_is_complete(struct edma_chan * chan, edma_cookie_t cookie)
{

    signed long last_issued = 0;
    signed long last_completed = 0;

    uint32_t error_value = EDMA_IN_PROGRESS;
    edma_addr_t physical_addr = 0;
    struct edma_descriptor_block * block = NULL;
    struct edma_descriptor_internal * desc_array = NULL;
    int32_t phys_addr_block = BLOCK_NOT_FOUND;
    int32_t i = 0;
    int32_t chan_csr_val = 0;

    /* Obtain the next descriptor address */
    uint32_t chan_cnt = chan->channel_no;
    uint32_t cdar = os_readl(get_mm_registers(chan_cnt,EDMA_CDAR));
    uint32_t cduar = os_readl(get_mm_registers(chan_cnt,EDMA_CDUAR));
    physical_addr = cduar;
    physical_addr = physical_addr << SHIFT_DWORD;
    physical_addr = physical_addr | cdar;

    last_completed = chan->completed_cookie;
    last_issued = chan->cookie;
    DMA_DEBUG_PRINT_OS_2("IS_COMPLETE - cookie %x last_completed %x\n", 
    (int)cookie, (int)last_completed);
    
   
    
    #ifdef __KERNEL__
    os_spin_lock_irq(&chan->desc_lock);
    #endif
    #ifdef __FreeBSD__
         os_spin_lock(&chan->desc_lock);
    #endif
       
    block = get_descriptor_block(chan->channel_no);

       while(block != NULL)
       {
        
            if(block != NULL)
            {
                desc_array = block->descriptor_pool;
            }

            if(desc_array != NULL)
            {
                if(cookie == block->last_cookie_err)
                {
                   
                   error_value = EDMA_ERROR;
                   break;
                }
            }
            else
            {
                 error_value = EDMA_NO_RECORD;
                 break;
            }
            if((physical_addr >= desc_array[0].phys)  && 
            (physical_addr <= desc_array[DEFAULT_DESCRIPTOR_NO - 1].phys))
            {               
                phys_addr_block = BLOCK_FOUND;
                
                for(i = 0; i <= (DEFAULT_DESCRIPTOR_NO - 1); i++)
                {
                    if(desc_array[i].phys == physical_addr)
                    {
                       chan_csr_val = os_readl(
                       get_mm_registers(chan->channel_no, EDMA_CSR));

                        if(CHAN_ACTIVE & chan_csr_val)
                        {
                            
                            if(i != 0)
                            {
                                 if(desc_array[i - 1].cookie < cookie)
                                 {
                                     last_completed = desc_array[i - 1].cookie;
                                 }
                            }
                            
                        }
                        else
                        {
                           last_completed = desc_array[i].cookie;
                           
                        }
                        break;
                   }
                }
                break;
             } 
             else
             {
                 block->status = OBSOLETE;
             }
        block = (struct edma_descriptor_block *)block->next_ptr;

    }/* while */
    #ifdef __KERNEL__
         os_spin_unlock_irq(&chan->desc_lock);
    #endif
    #ifdef __FreeBSD__
         os_spin_unlock(&chan->desc_lock);
    #endif



    if(error_value != EDMA_ERROR)
    {
        if(last_completed <= last_issued)
        {
            if(cookie  <= last_completed)
            {
                 error_value = EDMA_SUCCESS;
                 DMA_DEBUG_PRINT_OS_0(
                 "IS_COMPLETE - Last complete <= Last issued\n");
            }
 
        }
        else
        {
            if(cookie <= last_completed)
            {
                error_value = EDMA_SUCCESS;
                DMA_DEBUG_PRINT_OS_0(
                "IS_COMPLETE - Last complete > Last issued\n");
            }        
        }
    }

    if(chan->client->edma_completion_callback != NULL)
    { 
           
           if(error_value == EDMA_SUCCESS)
           {
               chan->client->edma_completion_callback(chan->client, chan,
                EDMA_TRANSACTION_COMPLETE, cookie);
           }
           else if(error_value == EDMA_ERROR)
           {
               chan->client->edma_completion_callback(chan->client, chan,
               EDMA_TRANSACTION_ERROR, cookie);
           }
    }

    return error_value;

}



/**
 ***************************************************************************** 
 *****************************************************************************/
uint32_t edma_get_errors(struct edma_chan * chan,
                         edma_cookie_t cookie)
{

    /*
     * 0 = No errors found, or there is no longer a record of that 
     * value in the descriptor list. In other words, 
     * the block associated with that descriptor is gone.
     *
     */
    uint32_t error_value = 0;
    struct edma_descriptor_block * block = NULL;
    
    
    DMA_DEBUG_PRINT_OS_0("EDMA_ERROR\n");

    #ifdef __KERNEL__
    os_spin_lock_irq(&chan->desc_lock);
    #endif
    #ifdef __FreeBSD__
         os_spin_lock(&chan->desc_lock);
    #endif
    
    block = get_descriptor_block(chan->channel_no);
    
    if(block != NULL)
    { 
    
    do
    {
       
            /* Check first for a cookie error? */
            if(cookie == block->last_cookie_err)
            {
                error_value = block->last_ferr_value;
                break;
            }
           
           
            block = 
            (struct edma_descriptor_block *) block->next_ptr; 
       }
       while(block != NULL);
    }
    

    #ifdef __KERNEL__
         os_spin_unlock_irq(&chan->desc_lock);
    #endif
    #ifdef __FreeBSD__
         os_spin_unlock(&chan->desc_lock);
    #endif

    return error_value;
}

/**
 ***************************************************************************** 
 *****************************************************************************/
int32_t edma_change_channel_priority(uint32_t channel)
{
    uint32_t dgcr_val, chan_csr_val, chan_ccr_val;
    int32_t chan_cnt;
    int32_t ret = EDMA_CP_FAIL;


     /* If any of the channels are in the midst of a transaction,
    pause them. Resume them after setting the channel priority.  */
    for(chan_cnt = 0; chan_cnt < MAX_NO_CHANS; chan_cnt++)
    {
       
        chan_csr_val = os_readl(get_mm_registers(chan_cnt, EDMA_CSR));
        
        if(chan_csr_val & CHAN_ACTIVE)
        {
            chan_ccr_val = os_readl(get_mm_registers(chan_cnt, EDMA_CCR));
            os_writel(chan_ccr_val | SUSPEND_DMA, 
            get_mm_registers(chan_cnt, EDMA_CCR));
           
        }
        
        do
        {

            chan_csr_val = os_readl(get_mm_registers(chan_cnt, EDMA_CSR));
         
        }while(!(chan_csr_val |  DMA_SUSPENDED));

    } /* FOR */
    
    
   
    dgcr_val = os_readl(get_mm_registers(0,EDMA_DCGC_REG));


    /* If channel value is 0-3 set the priority, else set to default. */
    if((channel >= 0) && (channel < MAX_NO_CHANS))
    {

        /* Set channel priority */
        os_writel(dgcr_val | channel | CHANNEL_PRIORITY_ENABLED,
        get_mm_registers(0, EDMA_DCGC_REG));
            
        ret = EDMA_CP_VALID_CHANNEL;
    }
    else
    {
        /* Set channel priority to the default arbitration scheme */
        if(dgcr_val & CHANNEL_PRIORITY_ENABLED)
        {
           dgcr_val= dgcr_val & CHANNEL_PRIORITY_DISABLED;
           os_writel(dgcr_val, get_mm_registers(0,EDMA_DCGC_REG));

        }
        ret = EDMA_CP_DEFAULT;
    }

    /* Resume */
    for(chan_cnt = 0; chan_cnt < MAX_NO_CHANS; chan_cnt++)
    { 

        chan_ccr_val = readl(get_mm_registers(chan_cnt, EDMA_CCR));

        if(chan_ccr_val & SUSPEND_DMA)
        {
            /* Clear the Suspend DMA bit and write the value back */
            os_writel(0, get_mm_registers(chan_cnt, EDMA_CCR));
            os_writel(RESUME_DMA, get_mm_registers(chan_cnt, EDMA_CCR));
            chan_ccr_val = readl(get_mm_registers(chan_cnt, EDMA_CCR));
            
            if(chan_ccr_val & SUSPEND_DMA)
            {
                ret = EDMA_CP_FAIL;
            }
        }
    }/* FOR */

    dgcr_val = os_readl(get_mm_registers(0,EDMA_DCGC_REG));
    

    return ret;
}








/**
 ***************************************************************************** 
 *****************************************************************************/
edma_cookie_t edma_memcpy(uint32_t memory_ops, struct edma_chan *chan, 
edma_addr_t dest, edma_addr_t src, size_t size)
{
    /* Variables for setting up transactions */
    struct edma_descriptor_internal * desc_array = NULL;
    uint32_t channel_no = 0;
    uint32_t chan_csr_val = 0;
    uint32_t len = 0;
    edma_cookie_t cookie = 0;
    uint32_t new_memory_ops = memory_ops;
    int32_t num_desc, desc_index, first_index, chain_index;
    num_desc = desc_index = first_index = chain_index = 0;
   
    
    /* Clear the first two bits. These shouldn't be set by the client. */
    new_memory_ops = new_memory_ops & CLEAR_MEMORY_OPS;
    /* Configure the first two bits properly. */
    new_memory_ops = new_memory_ops | INT_END_OF_CHAIN | INT_ABORT;
   
    if(chan != NULL)
    {
        if(chan->channel_no >= 0 && chan->channel_no < MAX_NO_CHANS)
        {
            channel_no = chan->channel_no;
        }
        else
        {
           return ERROR_BAD_CHANNEL;
        }
    }
    else
    {
        return ERROR_BAD_CHANNEL;
    }

    /* Calculate the number of descriptors needed. */
    num_desc = size / MAX_TRANSFER_PER_DESC;
    if((size % MAX_TRANSFER_PER_DESC) != 0)
    {
        size = size % MAX_TRANSFER_PER_DESC;
        num_desc++;
    }
    else
    {
        size = MAX_TRANSFER_PER_DESC;
    }

    desc_array = (struct edma_descriptor_internal *)get_descriptors(num_desc, 
    channel_no);
    DMA_DEBUG_PRINT_OS_1("Descriptor Array %x \n", (int)desc_array); 
    DMA_DEBUG_PRINT_OS_1("Number of Descriptors %x \n", num_desc);
    /* Incrementing Cookie */
    chan->cookie++;
    if(chan->cookie < 0)
    {
        chan->cookie  = 1;
    }
    cookie = chan->cookie;


    first_index = desc_index;
    
   /*
    *   Break up the transfer into blocks of MAX_TRANSER_PER_DESC or smaller.
    *   Each block is assigned one descriptor. Chain the descriptors together 
    *   in a linked list format.
    *
    */
    
     while(num_desc)
     {
        
        if(num_desc == 1)
        {
            desc_array[desc_index].hw.ndar  = 0;
            desc_array[desc_index].hw.nduar = 0;
            len = size;
            desc_array[desc_index].hw.tcr = len;
            DMA_DEBUG_PRINT_OS_1("Number_desc (should be one) %x\n", num_desc); 
            DMA_DEBUG_PRINT_OS_1("Descriptor Array %x\n",
            (int)desc_array[desc_index].phys); 
            DMA_DEBUG_PRINT_OS_1("Size %x\n",len); 
         
            
         
        }
        else
        {
           
            len = MAX_TRANSFER_PER_DESC;
            desc_array[desc_index].hw.ndar  = desc_array[desc_index].next_ndar;
            desc_array[desc_index].hw.nduar = desc_array[desc_index].next_nduar;
            desc_array[desc_index].hw.tcr =  len;

           

            /* Create an artificial ABORT Situation for testing */
            #ifdef ABORT_TEST_ON
                if(memory_ops == 0xFFFFFFFF)
                {
                    desc_array[desc_index].hw.ndar  = 
                    desc_array[desc_index].next_ndar + 1;
                    desc_array[desc_index].hw.nduar = 
                    desc_array[desc_index].next_nduar;
                    desc_array[desc_index].hw.dcr = (SRC_DWORD_ALIGNED |    \
                    DST_DWORD_ALIGNED | DST_COHERENT | SRC_COHERENT);

                }
            #endif

            DMA_DEBUG_PRINT_OS_1("Number_desc (should be many)%x\n", num_desc);
            DMA_DEBUG_PRINT_OS_1(
            "Descriptor Array %x\n",(int)desc_array[desc_index].phys); 
            DMA_DEBUG_PRINT_OS_1("Size %x\n",len); 

        } 

        
        /* 
         * This section will cause problems if the kernel is not
         * configured to deal with 64 bit values. See programmer's guide 
         * for details.
         */
         
        
        desc_array[desc_index].hw.sar = src & LOWER_64;
        desc_array[desc_index].hw.suar = (src >> SHIFT_DWORD) & LOWER_64;
        desc_array[desc_index].hw.dar = dest & LOWER_64;
        desc_array[desc_index].hw.duar = (dest >> SHIFT_DWORD) & LOWER_64;

        desc_array[desc_index].hw.dcr = new_memory_ops;
        desc_array[desc_index].cookie = cookie;
        
        desc_index++;
        if(desc_index == DEFAULT_DESCRIPTOR_NO)
        {
            desc_index = 0;
        }
        num_desc--;

        if((SRC_BUFFER_INIT & memory_ops) == 0)
        {
            if(SRC_DEC & memory_ops)
            {
                src = src - MAX_TRANSFER_PER_DESC;
           
            }
            else
            {
                src = src + MAX_TRANSFER_PER_DESC;
            }
        }

        if((DST_CONST & memory_ops) == 0)
        {
            dest = dest + MAX_TRANSFER_PER_DESC;
        }


    } 
    

    /* Check the status register. */

    chan_csr_val =
    os_readl(get_mm_registers(channel_no, EDMA_CSR));
   
    
    if(chan_csr_val & CHAN_ACTIVE)
    {
        /* 
        * Suspend the current DMA Operation and 
        * wait for the appropriate status value. 
        */
       
        DMA_DEBUG_PRINT_OS_0(CHANNEL_ACTIVE); 

        os_writel(SUSPEND_DMA, 
        get_mm_registers(channel_no, EDMA_CCR));
       
        do
        {
            
            chan_csr_val = os_readl(get_mm_registers(channel_no, EDMA_CSR));
            
        }
        while((chan_csr_val &  DMA_SUSPENDED) == 0);
       
        
        /* Append to the last batch of descriptors in the chain. */
        append_descriptors(channel_no, desc_array);
        /* Clear the Suspend and Resume the Operation. */
        
        os_writel(0, 
        get_mm_registers(channel_no, EDMA_CCR));
        os_writel(RESUME_DMA, 
        get_mm_registers(channel_no, EDMA_CCR)); 
        


    } 
    else
    {
            DMA_DEBUG_PRINT_OS_2("MEMCPY_COOKIE %x index %x\n",
            (int)desc_array[first_index].cookie, first_index);
            DMA_DEBUG_PRINT_OS_0(CHANNEL_IDLE); 
            os_writel(desc_array[first_index].phys & LOWER_64,
            get_mm_registers(channel_no, EDMA_NDAR));
            os_writel((desc_array[first_index].phys >> SHIFT_DWORD) & LOWER_64,
            get_mm_registers(channel_no, EDMA_NDUAR));
            os_writel(START_DMA, 
            get_mm_registers(channel_no, EDMA_CCR));
    
    }

    return cookie;

}
/**
 ***************************************************************************** 
 *****************************************************************************/
inline int32_t edma_set_transaction(struct edma_chan *chan, 
uint32_t memory_ops)
{
    if(chan)
    {
        chan->memory_ops = memory_ops;
        return 0;   
    }
    
    return -1;
}


