
/****************************************************************************
 * This file is provided under a dual BSD/GPLv2 license.  When using or 
 *   redistributing this file, you may do so under either license.
 * 
 *   GPL LICENSE SUMMARY
 * 
 *   Copyright(c) 2007,2008 Intel Corporation. All rights reserved.
 * 
 *   This program is free software; you can redistribute it and/or modify 
 *   it under the terms of version 2 of the GNU General Public License as
 *   published by the Free Software Foundation.
 * 
 *   This program is distributed in the hope that it will be useful, but 
 *   WITHOUT ANY WARRANTY; without even the implied warranty of 
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 *   General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License 
 *   along with this program; if not, write to the Free Software 
 *   Foundation, Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *   The full GNU General Public License is included in this distribution 
 *   in the file called LICENSE.GPL.
 * 
 *   Contact Information:
 *   Intel Corporation
 * 
 *   BSD LICENSE 
 * 
 *   Copyright(c) 2007,2008 Intel Corporation. All rights reserved.
 *   All rights reserved.
 * 
 *   Redistribution and use in source and binary forms, with or without 
 *   modification, are permitted provided that the following conditions 
 *   are met:
 * 
 *     * Redistributions of source code must retain the above copyright 
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright 
 *       notice, this list of conditions and the following disclaimer in 
 *       the documentation and/or other materials provided with the 
 *       distribution.
 *     * Neither the name of Intel Corporation nor the names of its 
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 * 
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 *   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 *   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 *   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * 
 *  version: Embedded.L.1.0.34
 *****************************************************************************/

/******************************************************************************
 * @ingroup EDMA_CLIENT_API
 *
 * @file dma_common.c
 *
 * @description
 * This group of functions comprises background functionality of Enhanced 
 * DMA driver.
 *****************************************************************************/


#include "dma_internals.h"
#include "os_list.h"
#include "os.h"

#ifdef __KERNEL__
#include <linux/delay.h>
#endif

#ifdef __FreeBSD__
#include <machine/bus.h>
bus_dma_tag_t bus_tag;
#endif

extern struct edma_mm_regs * g_mm_regs;
extern uint32_t g_channel_ccr_offsets[MAX_NO_CHANS];
extern struct os_list_start g_edma_client_list;

struct edma_descriptor_list g_descriptor_lists[MAX_NO_CHANS];
struct edma_chan * g_edma_channel_list[MAX_NO_CHANS];
os_spinlock_t g_channel_lock;




/***************************************************************************** 
 * Abstract
 * Wrapper function that returns memory mapped register.
 * Side Effects:
 * Assumptions:
 * Used internally only. 
 * Return Values: uint8_t * ptr to a memory mapped register.
 *****************************************************************************/
inline uint8_t* get_mm_registers(uint32_t channel, 
                                /* Channel associated with register block */
                                  uint32_t offset/* Offset of the register */)
{
    uint8_t* ret_val = ((uint8_t*)g_mm_regs + \
        g_channel_ccr_offsets[channel] + offset);

    return ret_val;
}

/***************************************************************************** 
 * Abstract
 * Wrapper function that returns edma_client_internal * handle.
 * Side Effects:
 * Assumptions:
 * The edma_client_internal structure is not to be confused with edma_client.
 * This function is not part of the client driver API or the application
 * interface.
 * Used internally only. 
 * Return Values: Nothing
 *****************************************************************************/
inline struct edma_client_internal * get_client_internal(
struct edma_client *client /* Client handle inside edma_client_internal */)
{
    struct edma_client_internal* ret_val = \
          (struct edma_client_internal *)((uint8_t*)client - \
            (HANDLE_NO * sizeof(uint8_t*)));
    
    return ret_val;

}


/***************************************************************************** 
 * Abstract
 * Returns a descriptor block per channel.
 * Side Effects:
 * Assumptions:
 * It is called internally.
 * Used internally only. 
 * Return Values: Nothing
 *****************************************************************************/
inline struct edma_descriptor_block * get_descriptor_block(
uint32_t channel /* Channel associated with register block */
)
{

    struct edma_descriptor_block * p_desc_block = NULL;

   
    p_desc_block =
    (struct edma_descriptor_block *)
    (g_descriptor_lists[channel].block_list.next_ptr);

    
    return p_desc_block;

}
/*****************************************************************************
 * Abstract
 * Initializes descriptors in a descriptor pool of memory per/channel.
 * Side Effects:
 * Assumptions:
 * Used internally only. 
 * Return Values: Nothing
 *****************************************************************************/
void edma_descriptors_init(struct edma_chan * chan, 
/* channel indicating which descriptor pool to initialize*/
struct edma_descriptor_block  * block
/* block to initialize */)
{
   struct edma_descriptor_internal * prev = NULL;
   struct edma_descriptor_internal * desc = block->descriptor_pool; 
   int32_t descriptors = 0;
   uint32_t handle = block->handle;

   /*
    * This loop simply sets up a block of free descriptors,
    * adjusting all of the pointers in individual descriptors to
    * point to the appropriate next and previous descriptors
    * in linked list fashion.
    *
    */

    for(descriptors = 0; descriptors < DEFAULT_DESCRIPTOR_NO; 
    descriptors++)
    {
        desc->phys = 
        handle + (descriptors * sizeof(struct edma_descriptor_internal));
        desc->cookie = INVALID_COOKIE;
        desc->block = block;
        
        if(descriptors != (DEFAULT_DESCRIPTOR_NO - 1))
        {
            desc++;
        }
    }

    desc = block->descriptor_pool; 

    for(descriptors=0; descriptors < DEFAULT_DESCRIPTOR_NO; 
    descriptors++)
    {
        prev = desc;
        if(descriptors != (DEFAULT_DESCRIPTOR_NO - 1))
        {
            desc++;    
            prev->next_ndar =  desc->phys & LOWER_64;
            prev->next_nduar = (desc->phys >> SHIFT_DWORD) & LOWER_64;
            
            if(desc->phys == 0)
            {
                DMA_DEBUG_PRINT_OS_0("BAD_ADDR\n");
                break;
                
            }
 
        }
    }

}





/***************************************************************************** 
 * Abstract
 * Returns a descriptor list to edma_memcpy.
 * Assumptions:
 * Searches the descriptor list for enough n number of requested descriptors.
 * Returns the starting pt in the descriptor array (desc_array[free_index]) 
 * where unused descriptors begin. If it cannot find enough descriptors in a 
 * descriptor block it creates a new block.
 * Used by the edma_memcpy function only. 
 * Return Values:
 * ptr to a descriptor_internal (first descriptor not being used in a block).
 *****************************************************************************/
struct edma_descriptor_internal * get_descriptors(int32_t num, 
/* number of descriptors requested */
int32_t channel) /* on which channel descriptors are requested */
{
    uint32_t start_index = 0;
    struct edma_descriptor_internal * desc_array = NULL;
    struct edma_descriptor_block * block = NULL;
    start_index = 0;
    
    DMA_DEBUG_PRINT_OS_0("Getting Descriptors...\n");
    
    block = get_last_block(channel);
   
    if(block != NULL)
    {
        

        if((DEFAULT_DESCRIPTOR_NO - block->free_index) >= num)
        {
                            
            start_index = block->free_index;
            desc_array = (struct edma_descriptor_internal *)
            block->descriptor_pool;
            block->free_index = block->free_index + num;
            DMA_DEBUG_PRINT_OS_0("Got exsisting block \n");    
            
        }
        /* If there are not enough desc available, create another block. */
        else
        {            
             DMA_DEBUG_PRINT_OS_0("Added block\n"); 
             block = add_new_block(channel);
             if(block)
             {
                 desc_array = (struct edma_descriptor_internal*)
                 block->descriptor_pool;
                 block->free_index = block->free_index + num;
             }
                    
        }
        
    } 
    /* There is an empty list, try to add a new block */
    else
    {
    
        DMA_DEBUG_PRINT_OS_0("Empty List \n");
        block = add_new_block(channel);
        if(block != NULL)
        {
            desc_array = (struct edma_descriptor_internal *)
            block->descriptor_pool;
            block->free_index = block->free_index + num;
        }
       
    }
    
    
    
    if(desc_array != NULL)
    {
        desc_array = &desc_array[start_index]; 
    }
    DMA_DEBUG_PRINT_OS_0("Returning a Descriptor \n"); 
    return desc_array; 
}

/***************************************************************************** 
 * Abstract
 * Creates structures and lists internal to driver operations.
 * Side Effects:
 * Assumptions:
 * Called at driver initialization time.
 * Return Values: SUCCESS (0), FAILED (-1)
 *****************************************************************************/
int32_t create_internal_structures(void)
{
    int32_t channel;
    int32_t err = SUCCESS;

    #ifdef __FreeBSD__
    int32_t err_tag = 0 ;
      /* Create DMA ready memory pools */
    if ((err_tag = bus_dma_tag_create(NULL, 
    sizeof(struct edma_descriptor_internal), 0,
    BUS_SPACE_MAXADDR_32BIT, BUS_SPACE_MAXADDR, NULL, NULL,
    F_PAGE, 1, F_PAGE, 0,
    NULL, NULL, &bus_tag)) != 0) 
    {
        DMA_DEBUG_PRINT_OS_1("EDMA: DMA TAG create %d", err_tag);
        return err_tag;
    }
    #endif
    
    /* Create and initialize the channel list */ 
    DMA_DEBUG_PRINT_OS_0(CREATING_INTERNAL_STRUCT);
    os_spin_lock_init(&g_channel_lock);
    for (channel = 0; channel < MAX_NO_CHANS; channel++)
    {
        g_edma_channel_list[channel] = 
        os_malloc(sizeof(struct edma_chan),
        STANDARD_MEM, GENERAL_KERNEL);
        
        DMA_DEBUG_PRINT_OS_1("CHANNEL ADDRESS %x\n", 
        (int)g_edma_channel_list[channel]);
      
        if(g_edma_channel_list[channel] != NULL)
        {
            
            memset(g_edma_channel_list[channel],
            0, sizeof(struct edma_chan));
           
            /* channels are initialized as not in use */
            g_edma_channel_list[channel]->sw_in_use = NOT_IN_USE;
            g_edma_channel_list[channel]->channel_no = channel;
            g_edma_channel_list[channel]->reg_offset = 
            g_channel_ccr_offsets[channel];
            
            /* Flip test */
            /*
            if(channel == 0)
            {
                g_edma_channel_list[0]->cookie = 0x4FFFFFFF;
                DMA_DEBUG_PRINT_OS_1("Cookie One  %i\n", 0x7FFFFFFF);
                DMA_DEBUG_PRINT_OS_1("Cookie Two  %i\n", 0x8FFFFFFF);
            } */
            
            os_spin_lock_init(&g_edma_channel_list[channel]->desc_lock);
            
        }
        else
        {
           return FAILED;
        }
    } /* FOR */

    /* Initialize DEFAULT_DESCRIPTOR_NO Descriptors Per Channel */
    for(channel = 0; channel < MAX_NO_CHANS; channel++)
    {
        g_descriptor_lists[channel].block_count = 0;

        if(add_new_block(channel) == NULL)
        {
            return FAILED;
        }
    }


    return err;
}




/***************************************************************************** 
 * Abstract
 * Frees structures and lists internal to driver operations.
 * Side Effects:
 * Assumptions:
 * Called at driver cleanup time.
 * Return Values: Nothing
 *****************************************************************************/
void free_internal_structures(void)
{
    int32_t channel;
    struct edma_descriptor_block  * block;
    int32_t size = 
    (sizeof(struct edma_descriptor_internal)  * DEFAULT_DESCRIPTOR_NO);
    struct edma_client_internal *client_internal = NULL;
    /* Free the malloced the channel list. */

    do
    {
       client_internal = (struct edma_client_internal *)
       os_list_remove_each_element_front(&g_edma_client_list);
       if(client_internal != NULL)
       {
           os_free(client_internal, STANDARD_MEM);
       }

    }while(client_internal != NULL);


    for (channel = 0; channel < MAX_NO_CHANS; channel++)
    {
        if(g_edma_channel_list[channel] != NULL)
        {
            os_free(g_edma_channel_list[channel], STANDARD_MEM);
        }

        do{
            block = remove_block(channel);    
            if(block != NULL)
            {
                #ifdef __FreeBSD__
                os_free_coherent((void*)bus_tag, size, block->pool_addr, 0);
                #endif
                #ifdef __KERNEL__
                os_free_coherent(NULL, size, 
                block->pool_addr, &(block->handle));
                os_free(block, STANDARD_MEM);
                #endif
            }

        }while(block != NULL);


    } /* FOR */

    #ifdef __FreeBSD__
    bus_dma_tag_destroy(bus_tag);
    #endif
}


/***************************************************************************** 
 * Abstract
 * Adds a new block to the list of exisiting blocks.
 * Side Effects:
 * Assumptions:
 * Used internally only. 
 * Return Values: Nothing
 *****************************************************************************/
struct edma_descriptor_block  * add_new_block(uint32_t channel)
{

    int32_t size = 
    (sizeof(struct edma_descriptor_internal)  * DEFAULT_DESCRIPTOR_NO);
    struct edma_descriptor_block  * new_block = 
    (struct edma_descriptor_block  *)
    os_malloc(
    sizeof(struct edma_descriptor_block),
    STANDARD_MEM, GENERAL_KERNEL);
    int32_t flags = GENERAL_KERNEL;

    memset(new_block, 0, sizeof(struct edma_descriptor_block));
    
    #ifdef __FreeBSD__
    
    flags = BUS_DMA_NOWAIT | BUS_DMA_ZERO;
    new_block->pool_addr = (uint8_t *)os_malloc_coherent((void*)bus_tag, 
    size, &(new_block->handle), 
    flags);
    #endif   
    
    #ifdef __KERNEL__
    new_block->pool_addr = 
    (uint8_t *)os_malloc_coherent(NULL, size, &(new_block->handle), 
    flags);
    #endif

    
   
    DMA_DEBUG_PRINT_OS_1("ADD_NEW_BLOCK  %x\n",
    (int)new_block);
    DMA_DEBUG_PRINT_OS_2("ADD_NEW_BLOCK phys %x pool %x\n", new_block->handle,
    (int)new_block->pool_addr);
    
    if(new_block->pool_addr != NULL)
    {
        memset(new_block->pool_addr, 0, size);
        new_block->descriptor_pool = 
       (struct edma_descriptor_internal *)new_block->pool_addr;     
    }
    
    
    if(new_block->descriptor_pool == NULL)
    {
        os_free(new_block, STANDARD_MEM);
        return NULL;
    }
    
    /* Initialize the free and used index per list. */
    new_block->last_ferr_value = -1;  
    new_block->last_cookie_err = -1;  
    new_block->last_desc_err = -1;    
    
    edma_descriptors_init(g_edma_channel_list[channel], new_block);
    
    #ifdef __KERNEL__
    os_spin_lock_irq(&g_edma_channel_list[channel]->desc_lock);
    #endif
    #ifdef __FreeBSD__
         os_spin_lock(&g_edma_channel_list[channel]->desc_lock);
    #endif
    
    os_list_insert_element_at_end((struct os_list_start *)new_block,
    &(g_descriptor_lists[channel].block_list));
    
    g_descriptor_lists[channel].block_count++;
    #ifdef __KERNEL__
    os_spin_unlock_irq(&g_edma_channel_list[channel]->desc_lock);
    #endif
    #ifdef __FreeBSD__
         os_spin_unlock(&g_edma_channel_list[channel]->desc_lock);
    #endif
    
    return new_block;
}

/***************************************************************************** 
 * Abstract
 * Removes a block from a list.
 * Side Effects:
 * Assumptions:
 * Used internally only. 
 * Return Values: Nothing
 *****************************************************************************/
struct edma_descriptor_block * remove_block(uint32_t channel)
{
    struct edma_descriptor_block  * block = NULL;
    
    
    block = (struct edma_descriptor_block *)
    os_list_remove_each_element_front(
    &(g_descriptor_lists[channel].block_list));
   
    g_descriptor_lists[channel].block_count--;
         
    return block;
    
    
    
}

/***************************************************************************** 
 * Abstract
 * Returns the last block in a list of blocks.
 * Side Effects:
 * Assumptions:
 * It is called internally.
 * Used internally only. 
 * Return Values: Nothing
 *****************************************************************************/
struct edma_descriptor_block * get_last_block(uint32_t channel)
{
    
    struct edma_descriptor_block * block = NULL;
    struct edma_descriptor_block * del_block = NULL;
    int32_t size = 
    (sizeof(struct edma_descriptor_internal)  * DEFAULT_DESCRIPTOR_NO);

    #ifdef __KERNEL__
    os_spin_lock_irq(&g_edma_channel_list[channel]->desc_lock);
    #endif
    #ifdef __FreeBSD__
         os_spin_lock(&g_edma_channel_list[channel]->desc_lock);
    #endif
    
    block =
    (struct edma_descriptor_block *)
    g_descriptor_lists[channel].block_list.next_ptr;
    
    
    /* Go to the end of the list. */
    if(block != NULL)
    {
        while(block->next_ptr != NULL)
        {
       
            if(block->status == OBSOLETE)
            {
               del_block = remove_block(channel);
                
               #ifdef __FreeBSD__

               os_free_coherent((void*)bus_tag, size, del_block->pool_addr, 0);

               #endif
               
               #ifdef __KERNEL__
               os_free_coherent(NULL,
               size, del_block->pool_addr, 
               &del_block->handle);
               #endif
               
               os_free(del_block, STANDARD_MEM);
           
               block = 
               (struct edma_descriptor_block *)
               g_descriptor_lists[channel].block_list.next_ptr;
            }
            else
            {
                block= (struct edma_descriptor_block *)block->next_ptr;
            }
  
        }
    }    

    
    #ifdef __KERNEL__
    os_spin_unlock_irq(&g_edma_channel_list[channel]->desc_lock);
    #endif
    #ifdef __FreeBSD__
         os_spin_unlock(&g_edma_channel_list[channel]->desc_lock);
    #endif
    
    /* Delete all of the obsolete blocks. */
  
    return block;
}

/***************************************************************************** 
 * Abstract
 * Appends descriptors to a chained list.
 * Side Effects:
 * Assumptions:
 * It is called internally.
 * Used internally only. 
 * Return Values: Nothing
 *****************************************************************************/
void append_descriptors(uint32_t channel, 
struct edma_descriptor_internal * desc_array)
{
    struct edma_descriptor_block * block = NULL;
    struct edma_descriptor_block * prev_block = NULL;
    struct edma_descriptor_internal * prev_desc = NULL;
    
    #ifdef __KERNEL__
    os_spin_lock_irq(&g_edma_channel_list[channel]->desc_lock);
    #endif
    #ifdef __FreeBSD__
    os_spin_lock(&g_edma_channel_list[channel]->desc_lock);
    #endif
    block = desc_array->block;

    /* We are at the beginning of a block. */
    if(block->descriptor_pool == desc_array)
    {
        prev_block = (struct edma_descriptor_block *)block->prev_ptr;

       
        if(prev_block != NULL)
        {
            prev_desc = (struct edma_descriptor_internal *)
            prev_block->descriptor_pool;
            prev_desc = &prev_desc[prev_block->free_index - 1];
            prev_desc->hw.ndar = (desc_array->phys & LOWER_64);
            prev_desc->hw.nduar = (desc_array->phys >> SHIFT_DWORD) & LOWER_64;

        }
    }
    /* We are in the middle of a block. */
    else
    {
        prev_desc =  desc_array - 1;
        
        prev_desc->hw.ndar = 
        prev_desc->next_ndar;
        prev_desc->hw.nduar = 
        prev_desc->next_nduar;
        
    }

    #ifdef __KERNEL__
    os_spin_unlock_irq(&g_edma_channel_list[channel]->desc_lock);
    #endif
    #ifdef __FreeBSD__
         os_spin_unlock(&g_edma_channel_list[channel]->desc_lock);
    #endif

}


/***************************************************************************** 
 * Abstract
 * Suspends the EDMA devices and cleans up data structures.
 * Side Effects:
 * Assumptions:
 * The operation assumes the EDMA engine finishes the current descriptor in 
 * chain, throwing away all remaining transactions.
 * It is called internally.
 * Used internally only. 
 * Return Values: Nothing
 *****************************************************************************/
int32_t edma_suspend_device(void)
{

     /* Check the status register. */
    int32_t wait_time, ret_val, chan_cnt;
    struct edma_chan * channel = NULL;
    struct edma_client * client = NULL;
    uint32_t chan_csr_val = 0;
    wait_time = ret_val = chan_cnt = 0;
   
    
    DMA_DEBUG_PRINT_OS_0(MESSAGE_SUSPEND);

    for (chan_cnt = 0; chan_cnt < MAX_NO_CHANS; chan_cnt++)
    {
        channel = g_edma_channel_list[chan_cnt];
        chan_csr_val = os_readl((uint8_t*)g_mm_regs 
        + g_channel_ccr_offsets[chan_cnt] + EDMA_CSR);
        DMA_DEBUG_PRINT_OS_1("EDMA: CSR: %x \n",
        chan_csr_val);    

        if(chan_csr_val & CHAN_ACTIVE)
        {
            
            /* 
             * Suspend the current DMA Operation,
             * which allows the edma engine to finish
             * the current descriptor. Wait
             * for the appropriate status value. 
             */
            
            DMA_DEBUG_PRINT_OS_0("EDMA: Channel Active  \n");
            
            os_writel(SUSPEND_DMA, (uint8_t*)g_mm_regs + 
            g_channel_ccr_offsets[chan_cnt]);

            do
            {
                chan_csr_val = os_readl((uint8_t*)g_mm_regs + 
                g_channel_ccr_offsets[chan_cnt] + EDMA_CSR);

                /* If we exceed a certain limit, we must break out */
                #ifdef __FreeBSD__
                    DELAY(1);
                #endif
 

                #ifdef __KERNEL__ 
                    udelay(PM_DELAY);
                #endif
                if(wait_time == EXCEED_LIMIT)
                {
                    return ERROR_SUSPEND_DMA;

                }
                wait_time++;

            }while((chan_csr_val &  DMA_SUSPENDED) == 0);

            /* Send a message to each client 
            that the transaction was interrupted */
            client = channel->client;
            if(client != NULL)
            {
                client->edma_callback(client, channel, EDMA_RESOURCE_SUSPEND);
            }

        }
 
    }

    free_internal_structures();
    
    return ret_val;
}

/***************************************************************************** 
 * Abstract
 * Reinitializes EDMA device's data structures.
 * Side Effects:
 * Assumptions:
 * It is called internally.
 * Used internally only. 
 * Return Values: Nothing
 *****************************************************************************/
void edma_resume_device(void)
{
     /* Check the status register. */
    uint32_t clear_bits;
    int32_t ret_val, chan_cnt;
    uint32_t chan_csr_val = 0;
    ret_val = chan_cnt = 0;
    
    for (chan_cnt = 0; chan_cnt < MAX_NO_CHANS; chan_cnt++)
    {

        
        chan_csr_val = os_readl((uint8_t*)g_mm_regs 
        + g_channel_ccr_offsets[chan_cnt] + EDMA_CSR);
        DMA_DEBUG_PRINT_OS_1("EDMA: CSR: %x \n",
            chan_csr_val);

        clear_bits = chan_csr_val & 
        (END_OF_CHAIN | END_OF_TRANSFER | DMA_ABORTED);
        if(clear_bits != 0)
        {
            os_writel(clear_bits, (uint8_t*)g_mm_regs + 
            g_channel_ccr_offsets[chan_cnt]  + EDMA_CSR);
        }  

        
    }
 
    create_internal_structures();

}








