/*****************************************************************************
 * This file is provided under a dual BSD/GPLv2 license.  When using or 
 *   redistributing this file, you may do so under either license.
 * 
 *   GPL LICENSE SUMMARY
 * 
 *   Copyright(c) 2007,2008 Intel Corporation. All rights reserved.
 * 
 *   This program is free software; you can redistribute it and/or modify 
 *   it under the terms of version 2 of the GNU General Public License as
 *   published by the Free Software Foundation.
 * 
 *   This program is distributed in the hope that it will be useful, but 
 *   WITHOUT ANY WARRANTY; without even the implied warranty of 
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 *   General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License 
 *   along with this program; if not, write to the Free Software 
 *   Foundation, Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *   The full GNU General Public License is included in this distribution 
 *   in the file called LICENSE.GPL.
 * 
 *   Contact Information:
 *   Intel Corporation
 * 
 *   BSD LICENSE 
 * 
 *   Copyright(c) 2007,2008 Intel Corporation. All rights reserved.
 *   All rights reserved.
 * 
 *   Redistribution and use in source and binary forms, with or without 
 *   modification, are permitted provided that the following conditions 
 *   are met:
 * 
 *     * Redistributions of source code must retain the above copyright 
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright 
 *       notice, this list of conditions and the following disclaimer in 
 *       the documentation and/or other materials provided with the 
 *       distribution.
 *     * Neither the name of Intel Corporation nor the names of its 
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 * 
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 *   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 *   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 *   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * 
 *  version: Embedded.L.1.0.34
 ****************************************************************************/
/*****************************************************************************
 * @ingroup EDMA_GENERAL
 *
 * @file os.c
 *
 * @description
 *   This is an internal API that intended as a lightweight common 
 *   portability layer that allows driver versions to reduce their 
 *   OS dependent code. 
 *
 ****************************************************************************/
#include "os.h"


#include <linux/types.h>
#include <linux/mm.h>
#include <linux/pci.h>
#include <asm/io.h>
/*****************************************************************************
NOTE: Because this code passes calls to OS APIs, incoming pointer values
are not checked. 
*****************************************************************************/
/**************************************************************************** 
 * Abstract
 * Allocates kernel allocated memory.
 * Side Effects:GFP_KERNEL
 * Assumptions:
 * FreeBSD has an extra parameter indicating a type of memory to be 
 * allocated.
 * This function is an effort to bypass that problem. 
 * Used internally only. 
 * Return Values: pointer to kernel allocated memory.
 ****************************************************************************/
inline void *os_malloc(
unsigned int size, /* size of memory chunk */
void *memory_type, /* types of memory */
unsigned int flags)
{
    return (void *) kmalloc(size, flags);
}


/*****************************************************************************
 * Abstract
 * Frees kernel allocated memory.
 * Side Effects:
 * Assumptions:
 * FreeBSD has an extra parameter indicating a type of memory to be allocated.
 * This function is an effort to bypass that problem. 
 * Used internally only. 
 * Return Values: NOTHING
 ****************************************************************************/
inline void os_free(
void *mem_ptr,/* pointer to memory region */
void *memory_type)   /* types of memory */
{ 
    kfree(mem_ptr); 
}

/*****************************************************************************
 * Abstract
 * Starts spinlock
 * Side Effects:
 * Assumptions:
 * Return Values: NOTHING
 ****************************************************************************/
inline void os_spin_lock(os_spinlock_t *spin /* Pointer to a spinlock */) 
{
    spin_lock(spin);
}

/*****************************************************************************
 * Abstract
 * Stops spinlock.
 * Side Effects:
 * Assumptions:
 * Return Values: NOTHING
 ****************************************************************************/
inline void os_spin_unlock(os_spinlock_t *spin /* Pointer to a spinlock */) 
{
    spin_unlock(spin);
}

/**************************************************************************** 
 * Abstract
 * Initializes spinlock.
 * Side Effects:
 * Assumptions:
 * Return Values: Initialized spinlock
 ****************************************************************************/
void os_spin_lock_init(os_spinlock_t *spin /* Pointer to a spinlock */) 
{
    spin_lock_init(spin);
}

/**************************************************************************** 
 * Abstract
 * Converts virtual kernel memory address to physical kernel memory address.
 * Side Effects:
 * Assumptions:
 * Used internally only. 
 * Return Values: Physical address 
 ****************************************************************************/
inline unsigned int os_virt_to_phys(void *ptr /* virtual address */)
{   
    return virt_to_phys(ptr);
}

/**************************************************************************** 
 * Abstract
 * Converts physical memory address to a virtual kernel address.
 * Side Effects:
 * Assumptions:
 * Used internally only. 
 * Return Values: Physical address 
 ****************************************************************************/
inline unsigned int os_phys_to_virt(unsigned int address
/* virtual address */)
{
    return (unsigned int) phys_to_virt(address);
}
/*****************************************************************************
 * Abstract
 * Writes to memory mapped I/O locations.
 * Side Effects:
 * Assumptions:
 * Used internally only. 
 * Return Values: NOTHING
 ****************************************************************************/
inline void os_writel(
unsigned int val,/* value to write */
void *addr /* pointer to a memory mapped addr */) 
{
    iowrite32(val, addr);
}

/*****************************************************************************
 * Abstract
 * Writes to memory mapped I/O locations.
 * Side Effects:
 * Assumptions:
 * Used internally only. 
 * Return Values: NOTHING
 ****************************************************************************/
inline void os_writew(
unsigned short val,/* value to write */
void *addr /* pointer to a memory mapped addr */)
{   
    iowrite16(val, addr);
}

/*****************************************************************************
 * Abstract
 * Writes to memory mapped I/O locations.
 * Side Effects:
 * Assumptions:
 * Used internally only. 
 * Return Values: NOTHING
 ****************************************************************************/
inline void os_writeb(
unsigned char val,/* value to write */
void *addr /* pointer to a memory mapped addr */) 
{
    iowrite8(val, addr);
}

 /****************************************************************************
 * Abstract
 * Reads from memory mapped I/O locations.
 * Side Effects:
 * Assumptions:
 * Used internally only. 
 * Return Values: NOTHING
 ****************************************************************************/
inline unsigned int os_readl(void *addr
/* pointer to a memory mapped addr */) 
{
    return ioread32(addr);
}
 /****************************************************************************
 * Abstract
 * Reads from memory mapped I/O locations.
 * Side Effects:
 * Assumptions:
 * Used internally only. 
 * Return Values: NOTHING
 ****************************************************************************/
inline unsigned int os_readw(void *addr 
/* pointer to a memory mapped addr */)  

{
    return ioread16(addr);
}
 /****************************************************************************
 * Abstract
 * Reads from memory mapped I/O locations.
 * Side Effects:
 * Assumptions:
 * Used internally only. 
 * Return Values: NOTHING
 ****************************************************************************/
inline unsigned int os_readb(void *addr 
/* pointer to a memory mapped addr */) 
{   
    return ioread8(addr);
}
/*****************************************************************************
 * Abstract
 * Starts spinlock
 * Side Effects:
 * Assumptions:
 * Return Values: NOTHING
 ****************************************************************************/
inline void os_spin_lock_irqsave(os_spinlock_t *spin, 
/* Pointer to a spinlock */ 
uint32_t flags /* irq spinlock requires flags*/)
{
    spin_lock_irqsave(spin, flags);
}
/*****************************************************************************
 * Abstract
 * Stops spinlock.
 * Side Effects:
 * AAssumptions: This type enables interrupts and restores state.
 * Return Values: NOTHING
 ****************************************************************************/
inline void os_spin_unlock_irqrestore(os_spinlock_t *spin,
/* Pointer to a spinlock */ 
uint32_t flags /* irq spinlock requires flags*/)
{
    spin_unlock_irqrestore(spin, flags);
}
/*****************************************************************************
 * Abstract
 * Starts spinlock. 
 * Side Effects:
 * Assumptions:This type of spinlock prevents HW interrupts from occurring.
 * Return Values: NOTHING
 ****************************************************************************/
inline void os_spin_lock_irq(os_spinlock_t * spin
/* Pointer to a spinlock */)
{
    spin_lock_irq(spin);
}
/*****************************************************************************
 * Abstract
 * Stops spinlock.
 * Side Effects:
 * AAssumptions: This type enables interrupts and restores state.
 * Return Values: NOTHING
 ****************************************************************************/
inline void os_spin_unlock_irq(os_spinlock_t * spin 
/* Pointer to a spinlock */)
{
    spin_unlock_irq(spin);
}
/*****************************************************************************
 * Abstract
 * Starts spinlock. 
 * Side Effects:
 * Assumptions:This type of spinlock prevents bottom half from executing.
 * Return Values: NOTHING
 ****************************************************************************/
inline void os_spin_lock_bh(os_spinlock_t * spin 
/* Pointer to a spinlock */)
{
    spin_lock_bh(spin);
}
/**************************************************************************** 
 * Abstract
 * Stops spinlock.
 * Side Effects:
 * AAssumptions: This type re-enables bottom half execution.
 * Return Values: NOTHING
 ****************************************************************************/
inline void os_spin_unlock_bh(os_spinlock_t * spin
/* Pointer to a spinlock */)
{
    spin_unlock_bh(spin);
}
/****************************************************************************
 * Abstract
 * writes out byte
 * Side Effects:
 * Assumptions:
 * Return Values: NOTHING
 ****************************************************************************/
inline void os_outb(uint32_t port, /* port number */
uint32_t value /* value to write */)
{
    outb(value, port);
}
/**************************************************************************** 
 * Abstract
 * Allocates DMA safe coherent memory.
 * Side Effects:
 * Assumptions:
 * Return Values: NOTHING
 ****************************************************************************/
inline void * os_malloc_coherent(void * device, /*pointer to a device */
size_t size, /* size to allocate */
int32_t * dma_handle, /* pointer to a handle */
int32_t gfp)
{

    void * ret = 0;
    
    ret = (uint8_t *)dma_alloc_coherent(device, size, 
    dma_handle, 
    gfp);
   

    return ret;

}
/**************************************************************************** 
 * Abstract
 *  Frees DMA safe coherent memory.
 * Side Effects:
 * Assumptions:
 * Return Values: NOTHING
 ****************************************************************************/
inline void os_free_coherent(void * device, /*pointer to a device */
size_t size, /* size to allocate */
void * dma,
int32_t * dma_handle /* pointer to a handle */)
{
    
    dma_free_coherent(NULL, size, 
    dma, *dma_handle); 
}
