/*****************************************************************************
 * This file is provided under a dual BSD/GPLv2 license.  When using or 
 *   redistributing this file, you may do so under either license.
 * 
 *   GPL LICENSE SUMMARY
 * 
 *   Copyright(c) 2007,2008 Intel Corporation. All rights reserved.
 * 
 *   This program is free software; you can redistribute it and/or modify 
 *   it under the terms of version 2 of the GNU General Public License as
 *   published by the Free Software Foundation.
 * 
 *   This program is distributed in the hope that it will be useful, but 
 *   WITHOUT ANY WARRANTY; without even the implied warranty of 
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 *   General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License 
 *   along with this program; if not, write to the Free Software 
 *   Foundation, Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *   The full GNU General Public License is included in this distribution 
 *   in the file called LICENSE.GPL.
 * 
 *   Contact Information:
 *   Intel Corporation
 * 
 *   BSD LICENSE 
 * 
 *   Copyright(c) 2007,2008 Intel Corporation. All rights reserved.
 *   All rights reserved.
 * 
 *   Redistribution and use in source and binary forms, with or without 
 *   modification, are permitted provided that the following conditions 
 *   are met:
 * 
 *     * Redistributions of source code must retain the above copyright 
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright 
 *       notice, this list of conditions and the following disclaimer in 
 *       the documentation and/or other materials provided with the 
 *       distribution.
 *     * Neither the name of Intel Corporation nor the names of its 
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 * 
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 *   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 *   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 *   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * 
 *  version: Embedded.L.1.0.34
 ****************************************************************************/
/*****************************************************************************
 * os_list.c
 *
 * Linked list data structure.
 * 
 * Description
 *      These functions create/destroy, manipulate a linked-list.
 * 
 ****************************************************************************/
/*****************************************************************************
 * @ingroup EDMA_GENERAL
 *
 * @file os_list.c
 *
 * @description
 * This file contains list-related functions.
 *
 ****************************************************************************/


#include "os.h"
#include "os_list.h"
 
#ifndef NULL
#define NULL    (void *)0
#endif

/**************************************************************************** 
* Lists in the driver, such as resource lists, are constructed with the 
* functions below and take the form described in the example:
*
* EXAMPLE:
*
*  ---- the list head prev should be NULL ----
*  HEAD.next_ptr = ELEMENT1
*  HEAD.prev_ptr = ELEMENT3
*
*  ELEMENT1.next_ptr = ELEMENT2
*  ELEMENT1.prev_ptr = HEAD
*
*  ELEMENT2.next_ptr = ELEMENT3
*  ELEMENT2.prev_ptr = ELEMENT1
*
*  ELEMENT3.next_ptr = NULL
*  ELEMENT3.prev_ptr = ELEMENT2
*
*****************************************************************************/
  
/*****************************************************************************
 * Abstract
 * Adjusts the pointers when a list element is removed by
 * the removal functions.
 * Side Effects:
 * Assumptions:
 * Used by the removal functions only.
 * Return Values: Nothing
 ****************************************************************************/
void os_list_adjust_ptrs(
struct os_list_start *prev,
/* pointer to the element before the element being removed */
struct os_list_start *head) /* pointer to the head of a list */
{
    
    struct os_list_start * next = NULL;

    if(head != NULL && prev != NULL)
    {
        next = (struct os_list_start *) prev->next_ptr;
        prev->next_ptr = next->next_ptr;
        
        if(prev == head && next->next_ptr == NULL) 
        {
            head->prev_ptr = NULL;
        }
        if(prev != head &&  next->next_ptr == NULL) 
        {
            head->prev_ptr = (unsigned char *) prev;
        }

        if(next->next_ptr != NULL) 
        {
            next = (struct os_list_start *) next->next_ptr;
            next->prev_ptr = (unsigned char *) prev;
        }
    
    }
    
}

/**************************************************************************** 
 * Abstract
 * Removes an element from a list.
 * Side Effects:
 * Assumptions:
 * Used by the removal functions only.
 * Return Values: Nothing
 ****************************************************************************/
void os_list_remove_element(
struct os_list_start *element,    /* list element to remove */
struct os_list_start *head
/* head of the list from which you wish to remove the element */
)
{
    /* Obtain the element before the element that is being removed. */
    struct os_list_start * prev = NULL;
    if(element != NULL)
    {
        prev = (struct os_list_start *) element->prev_ptr;
    }
    
    os_list_adjust_ptrs(prev, head);
}

/**************************************************************************** 
 * Abstract
 * Initializes list to proper empty values.
 * Side Effects:
 * Assumptions:
 * Return Values: Nothing
 ****************************************************************************/

void os_list_linked_list_reinit(struct os_list_start *list) 
/* pointer to the list that is being initialized */
{
    if(list != NULL) 
    {
        list->next_ptr = NULL;
        list->prev_ptr = NULL;
    }
}


/**************************************************************************** 
 * Abstract
 * Removes an element from the front of the list.
 * Side Effects:
 * Assumptions:
 * This function should be called within a loop
 * to free an entire list.
 * It is called internally.
 * Used internally only. 
 * Return Values: Pointer to the new head of the list
 ****************************************************************************/
unsigned char *os_list_remove_each_element_front(struct os_list_start * head) 
/* pointer to a list element that is being removed */
{   
    struct os_list_start * ret = NULL;
    struct os_list_start * next = NULL;

    if(head != NULL)
    {
        ret = (struct os_list_start *)head->next_ptr;
        if(ret != NULL)    
        {
            ret = (struct os_list_start *)head->next_ptr;
            next =(struct os_list_start *)ret->next_ptr;
            head->next_ptr = (unsigned char *)next;
            if(next != NULL) 
            {
                next->prev_ptr =(char *) head;
                if(next->next_ptr == NULL)
                {
                    head->prev_ptr = (unsigned char *)next;
                }
            }
            else 
            {
                head->next_ptr = head->prev_ptr = NULL;
            }
        }
    }

    return (unsigned char *) ret;

}

/**************************************************************************** 
 * Abstract
 * Inserts a list element at the front of the list.
 * Side Effects:
 * Assumptions:
 * Return Values: Nothing
 ****************************************************************************/
 void os_list_insert_element_front(
struct os_list_start *element,/* pointer to element to insert */
struct os_list_start *head)/* pointer to head of list*/
{
     struct os_list_start *temp = NULL;

     if(head != NULL && element != NULL)
     {
        if(head->next_ptr != NULL) 
        {
            temp = (struct os_list_start *) head->next_ptr;
            temp->prev_ptr = (unsigned char *) element;
        }
        else 
        {
            head->prev_ptr = (unsigned char *) element;
        }
        
        head->next_ptr = (unsigned char *) element;
        element->next_ptr = (unsigned char *) temp;
        element->prev_ptr = (unsigned char *) head;    
    }
}

/*****************************************************************************
 * Abstract
 * Inserts a list element at the end of the list.
 * Side Effects:
 * Assumptions: 
 * Return Values: Nothing
*****************************************************************************/
 void os_list_insert_element_at_end(
struct os_list_start *element,/* pointer to element to insert */
struct os_list_start *head)/* pointer to head of list*/
{

    struct os_list_start * last = NULL;
    
    if(head != NULL)
    {
        last = (struct os_list_start *) head->prev_ptr;
        if(last != NULL) 
        {
            
            last->next_ptr = (unsigned char *)element;
            element->prev_ptr = (unsigned char *)last;
            element->next_ptr = NULL;
            head->prev_ptr = (unsigned char *) element;
        }
        else 
        {
            
            head->next_ptr = (unsigned char *)element;
            head->prev_ptr = (unsigned char *)element;
            element->prev_ptr =(unsigned char *)head;
            element->next_ptr = NULL;
        }
    }
        
}




